from datetime import datetime
import json
from django.core.paginator import Paginator
from django.shortcuts import render
from chat_app.models import ChatRoom
from crea_app.serializers import CustomUserSerializer
from notifications_app.views import send_push_notification
from notifications_app.models import Notification
from rest_framework import viewsets, status
from django.db.models import Count
from save_app.models import SavedApprenticeship, SavedArtist, SavedCastingCall, SavedGigWork, SavedInternship
from posts_app.models import Like, Post
from social_app.models import Follow
from event.serializers import EventSerializer
from . serializers import AppliedApprenticeshipSerializer, AppliedInternshipSerializer, ApprenticeshipApplicationSerializer, ApprenticeshipApplicationStatusUpdateSerializer, ApprenticeshipEditSerializer, ApprenticeshipStatusSerializer, CastingCallStatusSerializer, CollabUserSerializer, CreateProjectSerializer, GigWorkApplicationStatusUpdateSerializer, GigworkStatusSerializer, HeadshotSerializer, InternshipApplicationSerializer, InternshipApplicationStatusUpdateSerializer, InternshipEditSerializer, InternshipStatusSerializer, InternshipViewSerializer, MediaReleaseSerializer, PressReleaseSerializer, UserApplicationSerializer, UserInternshipApplicationSerializer, UserSerializer
from django.db.models import Q
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status, generics
from rest_framework.permissions import IsAuthenticated
from rest_framework.parsers import MultiPartParser, FormParser
from django.contrib.auth import get_user_model
from .models import  Credit, GigWork,CastingCall, Headshot, MediaRelease, PressRelease,Project
from .serializers import GigWorkSerializer, CastingCallSerializer, ProjectSerializer
from rest_framework.response import Response
from rest_framework.decorators import api_view
from drf_yasg.utils import swagger_auto_schema
from .models import EventRegistration
from rest_framework.authentication import TokenAuthentication
from .serializers import EventRegistrationSerializer, FeedbackSerializer
from rest_framework_simplejwt.authentication import JWTAuthentication
from .serializers import ProjectTitleSerializer,ProjectUpdateSerializer,CommentSerializer
from rest_framework.parsers import MultiPartParser, FormParser
from .models import Feedback,ProjectComment
from rest_framework.exceptions import NotFound,PermissionDenied
from django.shortcuts import get_object_or_404
from crea_app.models import CustomUser as cuser, PrivacySettings
from save_app.models import SavedProject as save_project
from .models import Project, Team, TeamMember,ProjectReport,SavedProject,FollowedProject
from .serializers import ProjectUpdateSerializer,ProjectReportSerializer,ProjectPhotoUploadSerializer,ProjectVideoUploadSerializer
from .models import CastingCall,Award,CastingCallApplication,GigWorkApplication
from .serializers import AwardSerializer,GigWorkApplicationSerializer,SuggestedUserSerializer, ApprenticeshipSerializer, InternshipSerializer
from . serializers import TeamMemberSerializer  ,TeamSerializer,TeamJoinRequestSerializer,CastingCallApplicationSerializer
from userprofile_app.models import City, Profile, Skill, State
from django.db.models import F
from rest_framework.decorators import action
CustomUser = get_user_model()
from event.models import Event
from .models import ApprenticeApplication, Internship, InternshipApplication
# from .serializers import 
from .serializers import ApprenticeshipViewSerializer
from .models import Apprenticeship
from rest_framework.exceptions import ValidationError

class HeadshotUploadView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        headshot_obj, created = Headshot.objects.get_or_create(user=request.user)

        serializer = HeadshotSerializer(headshot_obj, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()

            # Return the file's URL after saving
            headshot_url = serializer.instance.headshot.url if serializer.instance.headshot else None

            return Response({
                "status": "success",
                "message": "Headshot uploaded successfully.",
                "data": {"headshot_url": headshot_url}
            }, status=status.HTTP_200_OK)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

class RemoveHeadshotView(APIView):
    permission_classes = [IsAuthenticated]

    def delete(self, request):
        try:
            # Get the user's Headshot instance
            headshot_obj = Headshot.objects.get(user=request.user)

            # Remove the file from storage
            if headshot_obj.headshot:
                headshot_obj.headshot.delete(save=False)

            # Remove the database entry
            headshot_obj.delete()

            return Response({
                "status": "success",
                "message": "Headshot removed successfully."
            }, status=status.HTTP_200_OK)

        except Headshot.DoesNotExist:
            return Response({"error": "Headshot not found."}, status=status.HTTP_404_NOT_FOUND)

class SuggestUserAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        user = request.user

        try:
            # Get the profile of the logged-in user
            profile = Profile.objects.get(user=user)
            follower_ids = Follow.objects.filter(following=user).values_list('follower__id', flat=True)
            following_ids = Follow.objects.filter(follower=user).values_list('following__id', flat=True)
            
            # Get the industries selected by the user
            selected_industry_ids = profile.selected_industries.values_list('id', flat=True)
            print(selected_industry_ids)

            if not selected_industry_ids and not selected_industry_ids:
                return Response({
                    "error": "failed",
                    "message": "The logged-in user has not selected any industries and No followers"
                }, status=400)

            # Query profiles with matching selected_primary_industry
            users = Profile.objects.filter(
                selected_primary_industry__id__in=selected_industry_ids
            ).exclude(user=user).select_related('user', 'selected_primary_industry').order_by('?')

            follower_profiles = Profile.objects.filter(
                user__id__in=follower_ids
            ).exclude(user__id__in=following_ids).select_related('user').order_by('?')

            # # Handle case where no users are found
            # if not users.exists():
            #     return Response({
            #         "error": "failed",
            #         "message": "No users found with similar industries."
            #     }, status=404)

            # Serialize and return user data
            serializer = SuggestedUserSerializer(users, many=True, context={'request': request})
            follower_serializer = SuggestedUserSerializer(follower_profiles, many=True, context={'request': request})
            return Response({
                "status": "success",
                "users": serializer.data,
                "followers": follower_serializer.data
            }, status=200)

        except Profile.DoesNotExist:
            return Response({
                "error": "failed",
                "message": "Profile for the logged-in user does not exist."
            }, status=404)

        except Exception as e:
            return Response({
                "error": "failed",
                "message": f"An unexpected error occurred: {str(e)}"
            }, status=500)

class CollabAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        user = request.user

        try:
            profile = Profile.objects.get(user=user)
            
            # Get the industries selected by the user
            selected_industry_ids = profile.selected_industries.values_list('id', flat=True)

            if not selected_industry_ids:
                return Response({
                    "error": "failed",
                    "message": "The logged-in user has not selected any industries"
                }, status=400)

            # Query profiles with matching selected_primary_industry
            users = Profile.objects.filter(
                selected_primary_industry__id__in=selected_industry_ids
            ).exclude(user=user).select_related('user', 'selected_primary_industry').order_by('?')

            page = request.GET.get('page', 1)  # Get page number, default to 1
            page_size = request.GET.get('page_size', 10)  # Get page size, default to 10
            paginator = Paginator(users, page_size)
            paginated_users = paginator.get_page(page)


            serializer = CollabUserSerializer(paginated_users , many=True)
            for i in range(len(serializer.data)):
                user_profile = users[i]
                is_saved = SavedArtist.objects.filter(user=request.user, artist_profile=user_profile).exists()
                serializer.data[i]["is_saved"] = is_saved
                user_id = serializer.data[i].get('user_id')
                print("user_id",user_id)

                chatroom = ChatRoom.objects.filter(
                Q(owner=request.user.id, members=user_id) |
                Q(owner=user_id, members=request.user.id)
                ).first()

                if chatroom:
                    serializer.data[i]["conversation_exists"] = True
                    serializer.data[i]["conversation_id"] = chatroom.conversation_id
                    serializer.data[i]["chatroom_id"] = chatroom.id
                else:
                    serializer.data[i]["conversation_exists"] = False
                

            return Response({
                "status": "success",
                "users": serializer.data,
                "current_page": paginated_users.number,
                "total_pages": paginator.num_pages,
                "total_items": paginator.count,
            }, status=200)

        except Profile.DoesNotExist:
            return Response({
                "error": "failed",
                "message": "Profile for the logged-in user does not exist."
            }, status=404)

        except Exception as e:
            return Response({
                "error": "failed",
                "message": f"An unexpected error occurred: {str(e)}"
            }, status=500)

class AddGigWorkAPIView(APIView):
    permission_classes=[IsAuthenticated]

    def post(self, request):
        serializer = GigWorkSerializer(data=request.data, context={"request": request})
        if serializer.is_valid():
            gigwork = serializer.save()
            return Response({
                "status": "success",
                "message": "GigWork added successfully.",
                "data": {
                    "gigworks_id": gigwork.id,
                }
            }, status=status.HTTP_201_CREATED)
        return Response({
            "status": "error",
            "message": "Invalid data",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class ListGigWorkAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        """
        List all available GigWork entries
        """
        # Fetch all GigWork objects from the database
        gigworks = GigWork.objects.all()

        # Serialize the data
        serializer = GigWorkSerializer(gigworks, many=True)  # many=True allows serialization of multiple objects
        return Response({
            "status": "success",
            "message": "GigWork list retrieved successfully.",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

class GetGigWorkAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, gigwork_id, *args, **kwargs):
        try:
            # Fetch the GigWork object based on the given ID
            print(request.user.id)
            gigwork = GigWork.objects.get(id=gigwork_id)
            # Serialize the data
            formatted_experience=[]
            serializer = GigWorkSerializer(gigwork, context={'request': request})
            save_status = SavedGigWork.objects.filter(gig_work = gigwork_id, user_id = request.user.id ).exists()
            applied_status = GigWorkApplication.objects.filter(gigwork_id = gigwork_id, user = request.user.id).exists()
            save_count = SavedGigWork.objects.filter(user_id = request.user.id ).count()
            experience = serializer.data.get('required_experience')
            created_by = True
            created_by_user = serializer.data.get('user')
            if created_by_user == request.user.id:
                created_by = True
            else:
                created_by= False
            
            for i in experience:
                formatted_experience.append( str(i) +" years of experience")
            # print(formatted_experience)
            return Response({
                "status": "success",
                "data": serializer.data,
                "save_status":save_status,
                "applied_status":applied_status,
                "saved_count":save_count,
                "formatted_experience":formatted_experience,
                "created_by_me":created_by
            }, status=status.HTTP_200_OK)
        except GigWork.DoesNotExist:
            return Response({
                "status": "error",
                "message": "GigWork not found"
            }, status=status.HTTP_404_NOT_FOUND)


class EditGigWorkAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, gigworks_id, *args, **kwargs):
        """
        Edit a specific GigWork by ID
        """
        try:
            # Fetch the instance by gigwork id
            gigwork = GigWork.objects.get(pk=gigworks_id)
        except GigWork.DoesNotExist:
            return Response({
                "status": "error",
                "message": "GigWork not found"
            }, status=status.HTTP_404_NOT_FOUND)

        # Pass the data to the serializer for validation
        serializer = GigWorkSerializer(gigwork, data=request.data, partial=True)  # partial=True allows partial updates
        if serializer.is_valid():
            serializer.save()  # Save changes
            return Response({
                "status": "success",
                "message": "GigWork updated successfully.",
                "data": {
                    "gigworks_id": gigwork.id,
                    "updated_at": gigwork.updated_at if hasattr(gigwork, 'updated_at') else None
                }
            }, status=status.HTTP_200_OK)

        return Response({
            "status": "error",
            "message": "Invalid data",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class DeleteGigWorkAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def delete(self, request, gigworks_id, *args, **kwargs):
        """
        Delete a specific GigWork by ID
        """
        try:
            # Fetch the instance by gigwork id
            gigwork = GigWork.objects.get(pk=gigworks_id)
        except GigWork.DoesNotExist:
            return Response({
                "status": "error",
                "message": "GigWork not found"
            }, status=status.HTTP_404_NOT_FOUND)

        # Perform deletion
        gigwork.delete()
        return Response({
            "status": "success",
            "message": "GigWork deleted successfully."
        }, status=status.HTTP_200_OK)

class ApplyGigWorkAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, *args, **kwargs):
        # Check if gigwork exists
        gigwork_id = request.data.get('gigworks_id')
        try:
            gigwork = GigWork.objects.get(id=gigwork_id)
        except GigWork.DoesNotExist:
            return Response({
                "status": "error",
                "message": "GigWork not found"
            }, status=status.HTTP_404_NOT_FOUND)

        # Prevent duplicate application logic
        if GigWorkApplication.objects.filter(user=request.user, gigwork=gigwork).exists():
            return Response({
                "status": "error",
                "message": "You have already applied to this GigWork"
            }, status=status.HTTP_400_BAD_REQUEST)
        data = {
            **request.data,
            "gigwork": gigwork.id  # Pass the gigwork ID directly
        }   

        # Serialize and validate data
        serializer = GigWorkApplicationSerializer(data=data)
        if serializer.is_valid():
            serializer.save(user=request.user, gigwork=gigwork)  # Attach authenticated user & gigwork instance
            return Response({
                "status": "success",
                "message": "Your application has been submitted successfully.",
                "data": {
                    "application_id": serializer.instance.id,
                    "applied_at": serializer.instance.applied_at
                }
            }, status=status.HTTP_201_CREATED)

        # If serializer fails, return validation errors
        return Response({
            "status": "error",
            "message": "Invalid application data",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class ListAppliedUsersAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, gigwork_id, *args, **kwargs):
        # Ensure the gigwork exists
        try:
            gigwork = GigWork.objects.get(id=gigwork_id)
        except GigWork.DoesNotExist:
            return Response({
                "status": "error",
                "message": "GigWork not found"
            }, status=status.HTTP_404_NOT_FOUND)

        # Query all applications for this gigwork
        applications = GigWorkApplication.objects.filter(gigwork=gigwork)

        # Serialize application data
        data = [
            {
                "id": application.id,
                "username": application.user.username,
                "full_name":application.user.full_name,
                "email": application.user.mobile_or_email,
                "user_message": application.user_message,
                "attachments": application.attachments,
                "applied_at": application.applied_at,
                "status": application.status,
            }
            for application in applications
        ]

        return Response({
            "status": "success",
            "message": "List of applicants retrieved successfully.",
            "data": data,
        }, status=status.HTTP_200_OK)


class UpdateGigworkStatusView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, gigwork_id, *args, **kwargs):
        gigwork = get_object_or_404(GigWork, pk=gigwork_id)
        serializer = GigworkStatusSerializer(instance=gigwork, data=request.data, partial=True)

        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Gigwork status updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "status": "error",
            "message": "Failed to update gigwork status.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)
    
class UpdateGigworkApplicationStatusView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, gigwork_id):
        data = request.data
        user_id = data.get('user_id', None)

        try:
            application = GigWorkApplication.objects.get(user=user_id, gigwork_id=gigwork_id)
        except GigWorkApplication.DoesNotExist:
            return Response({
                "success": False,
                "message": "Gigwork application not found."
            }, status=status.HTTP_404_NOT_FOUND)

        # Serialize and validate the input
        serializer = GigWorkApplicationStatusUpdateSerializer(application, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            Notification.objects.create(
                user=application.user,  # Notify the applicant
                action_type="applicant_status",
                message=f"Your application for the gig work '{application.gigwork.title}' has been updated. Current status: {application.status}."
            )
            if application.user.privacy_settings.enable_notifications:
                response = send_push_notification(
                    fcm_token=application.user.fcm_token,
                    title="Application status",
                    body=f"Your application for the gig work '{application.gigwork.title}' has been updated. Current status: {application.status}."
                )
            else:
                response = None 
            return Response({
                "success": True,
                "push_notification": response,
                "message": "Gig work application status updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "success": False,
            "message": "Invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class RemoveCreditAPIView(APIView):
   
    permission_classes = [IsAuthenticated]

    def post(self, request, projectId, userId):
        try:
            
            project = Project.objects.get(id=projectId)
            
           
            if project.user != request.user:
                raise PermissionDenied("You do not have permission to modify this project.")
           
            team_member = TeamMember.objects.get(user_id=userId, team__project=project)
           
            team_member.role = "No Credit"  
            team_member.save()
            
            return Response({"message": "Credit removed successfully."}, status=200)
        
        except Project.DoesNotExist:
            raise NotFound("Project not found.")
        
        except TeamMember.DoesNotExist:
            raise NotFound("Team member not found.")
        
        except Exception as e:
            return Response({"error": str(e)}, status=500)
        
# class ProjectPhotoUploadView(APIView):
#     permission_classes = [IsAuthenticated]

#     def post(self, request, projectId):
#         try:
#             project = Project.objects.get(id=projectId)
#         except Project.DoesNotExist:
#             return Response(
#                 {"detail": "Project not found."}, 
#                 status=status.HTTP_404_NOT_FOUND
#             )

#         serializer = ProjectPhotoUploadSerializer(instance=project, data=request.data, partial=True)
#         if serializer.is_valid():
#             serializer.save()
#             return Response(
#                 {"message": "Photo uploaded successfully.", "data": serializer.data}, 
#                 status=status.HTTP_200_OK
#             )
#         return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
#     def get(self, request, projectId):
#         try:
#             # Retrieve the project by its ID
#             project = Project.objects.get(id=projectId)
#         except Project.DoesNotExist:
#             return Response(
#                 {"detail": "Project not found."},
#                 status=status.HTTP_404_NOT_FOUND
#             )
        
#         # Serialize the project instance
#         # serializer = ProjectPhotoUploadSerializer(instance=project)
        
#         return Response(
#             {"status": "success",
#             "message": "images retrieved successfully",
#             "data": project.press_releases  },
#             status=status.HTTP_200_OK
#         )
class ProjectPhotoUploadView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, projectId):
        try:
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response(
                {"detail": "Project not found."},
                status=status.HTTP_404_NOT_FOUND
            )

        images = request.FILES.getlist('images')
        print(images)

        if not images:
            return Response(
                {"detail": "No images provided."},
                status=status.HTTP_400_BAD_REQUEST
            )

        # Create PressRelease instances for each image uploaded
        press_releases = []
        for image in images:
            press_release = PressRelease.objects.create(
                project=project,  # Associate each image with the correct project
                image=image
            )
            press_releases.append(press_release)

        # Serialize the press release objects and return a response
        serializer = PressReleaseSerializer(press_releases, many=True)
        return Response(
            {"success": True,"message": "Photos uploaded successfully.", "data": serializer.data},
            status=status.HTTP_201_CREATED
        )
    
class ProjectPhotoListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, projectId):
        try:
            # Fetch the project by its ID
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response(
                {"detail": "Project not found."},
                status=status.HTTP_404_NOT_FOUND
            )

        # Retrieve all press releases associated with the project
        press_releases = PressRelease.objects.filter(project=project)

        # Serialize the press releases data
        serializer = PressReleaseSerializer(press_releases, many=True)

        return Response(
            {"success": True, "message": "Press releases retrieved successfully.", "data": serializer.data},
            status=status.HTTP_200_OK
        )

class ProjectMediaUploadView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, projectId):
        try:
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response(
                {"detail": "Project not found."},
                status=status.HTTP_404_NOT_FOUND
            )

        files = request.FILES.getlist('files')  # Access multiple files from the request

        if not files:
            return Response(
                {"detail": "No files provided."},
                status=status.HTTP_400_BAD_REQUEST
            )

        # Create MediaRelease instances for each uploaded file
        media_releases = []
        for file in files:
            media_release = MediaRelease.objects.create(
                project=project,  # Associate each file with the correct project
                file=file
            )
            media_releases.append(media_release)

        # Serialize the media release objects and return a response
        serializer = MediaReleaseSerializer(media_releases, many=True)
        return Response(
            {"success": True, "message": "Media uploaded successfully.", "data": serializer.data},
            status=status.HTTP_201_CREATED
        )
    
class ProjectMediaListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, projectId):
        try:
            # Fetch the project by its ID
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response(
                {"detail": "Project not found."},
                status=status.HTTP_404_NOT_FOUND
            )

        # Retrieve all press releases associated with the project
        press_releases = MediaRelease.objects.filter(project=project)

        # Serialize the press releases data
        serializer = MediaReleaseSerializer(press_releases, many=True)

        return Response(
            {"success": True, "message": "Media releases retrieved successfully.", "data": serializer.data},
            status=status.HTTP_200_OK
        )
    

class FollowProjectView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, projectId):
        user = request.user
        try:
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response(
                {"detail": "Project not found."}, 
                status=status.HTTP_404_NOT_FOUND
            )

        follow_entry, created = FollowedProject.objects.get_or_create(user=user, project=project)
        if created:
            return Response(
                {"message": "You are now following the project."}, 
                status=status.HTTP_201_CREATED
            )
        else:
            # If already following, unfollow
            follow_entry.delete()
            return Response(
                {"message": "You have unfollowed the project."}, 
                status=status.HTTP_200_OK
            )



class SaveProjectView(APIView):
    def post(self, request, projectId):
        try:
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response({'error': 'Project not found'}, status=status.HTTP_404_NOT_FOUND)

        if SavedProject.objects.filter(user=request.user, project=project).exists():
            return Response({'message': 'Project is already saved'}, status=status.HTTP_200_OK)

   
        SavedProject.objects.create(user=request.user, project=project)
        return Response({'message': 'Project saved successfully'}, status=status.HTTP_201_CREATED)

    def delete(self, request, projectId):
        try:
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response({'error': 'Project not found'}, status=status.HTTP_404_NOT_FOUND)

        saved_project = SavedProject.objects.filter(user=request.user, project=project)
        if saved_project.exists():
            saved_project.delete()
            return Response({'message': 'Project unsaved successfully'}, status=status.HTTP_200_OK)

        return Response({'message': 'Project is not saved'}, status=status.HTTP_400_BAD_REQUEST)

class ReportProjectView(APIView):
    def post(self, request, projectId):
        try:
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response({'error': 'Project not found'}, status=status.HTTP_404_NOT_FOUND)

        serializer = ProjectReportSerializer(data=request.data)
        if serializer.is_valid():
            report, created = ProjectReport.objects.update_or_create(
                user=request.user,
                project=project,
                defaults={
                    'reason': serializer.validated_data['reason'],
                    'description': serializer.validated_data.get('description', ''),
                    # 'is_blocked': serializer.validated_data['is_blocked'],
                }
            )
            message = "Report submitted successfully"
            if report.is_blocked:
                message = "Project blocked successfully"

            return Response({'message': message}, status=status.HTTP_201_CREATED)
    
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    

    def get(self, request, projectId):
        try:
            project = Project.objects.get(id=projectId)
        except Project.DoesNotExist:
            return Response({'error': 'Project not found'}, status=status.HTTP_404_NOT_FOUND)

        # Get the report for the project, if it exists
        report = ProjectReport.objects.filter(project=project, user=request.user).first()

        if report:
            serializer = ProjectReportSerializer(report)
            return Response({'data': serializer.data}, status=status.HTTP_200_OK)
        else:
            return Response({'error': 'No report found for this project'}, status=status.HTTP_404_NOT_FOUND)
  
class ProjectCommentsView(APIView):
    permission_classes = [IsAuthenticated]
    serializer_class = CommentSerializer

    def get(self, request, project_id):
   
        project = get_object_or_404(Project, pk=project_id)
       
        comments = ProjectComment.objects.filter(project=project).order_by('-created_at')
       
        serializer = CommentSerializer(comments, many=True, context={'request': request})
        return Response({
            "status": "success",
            "message": "Comments retrieved successfully",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def post(self, request, project_id):
        """Add a comment to a specific project."""
        project = get_object_or_404(Project, pk=project_id)
        data = request.data.copy()
        data['project'] = project.id
        data['user'] = request.user.id  
        serializer = CommentSerializer(data=data)
        if serializer.is_valid():
            serializer.save()  
            return Response({
                "status": "success",
                "message": "Comment added successfully",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)
        
        return Response({
            "status": "error",
            "message": "Failed to add comment",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)
class ProjectCommentDeleteView(APIView):

    permission_classes =[IsAuthenticated]
    
    def delete(self,request,project_id,comment_id):
        user= request.user
        
        project = get_object_or_404(Project, pk=project_id)
        comment=ProjectComment.objects.filter(id=comment_id,project=project)
        comment = get_object_or_404(ProjectComment, pk=comment_id, project=project, user=user)
        comment.delete()
        return Response(
            {"messege":"comment deleted sucessfully"},
            status = status.HTTP_200_OK
        )  
class UpdateProjectView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    def put(self, request, project_id):
        project = get_object_or_404(Project, id=project_id)
        if project.user.id != request.user.id:
            return Response({
                "status": "error",
                "message": "You do not have permission to edit this project."
            }, status=status.HTTP_403_FORBIDDEN)

  
        serializer = ProjectUpdateSerializer(project, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Project details updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        return Response({
            "status": "error",
            "message": "Invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class ProjectDeleteView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    def delete(self, request, project_id):
        # Get the project object or return 404 if not found
        project = get_object_or_404(Project, id=project_id)

        # Check if the user has permission to delete the project
        if project.user.id != request.user.id:
            return Response({
                "status": "error",
                "message": "You do not have permission to delete this project."
            }, status=status.HTTP_403_FORBIDDEN)

        # Delete the project
        project.delete()

        return Response({
            "status": "success",
            "message": "Project deleted successfully."
        }, status=status.HTTP_200_OK)


        
class ProjectTitleListView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]

    def get(self, request, user_id=None):
        if user_id:
            # Retrieve all projects associated with the user
            projects = Project.objects.filter(user_id=user_id)
            if projects.exists():
                serializer = ProjectTitleSerializer(projects, many=True)
                return Response({"project_titles": serializer.data}, status=200)
            else:
                return Response({"message": "No projects found for this user."}, status=404)
        return Response({"message": "User ID is required."}, status=400)
    
class ProjectDetailView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]

    def get(self, request, project_id=None):
        print(request.user.id)
        if project_id:
            try:
                # Retrieve the specific project for the authenticated user
                project = Project.objects.get(id=project_id)
                serializer = ProjectSerializer(project, context={'request': request})  # Serialize a single project
                save_count = save_project.objects.filter(user_id = request.user.id ).count()
                save_status = save_project.objects.filter(project = project_id, user_id = request.user.id ).exists()
                created_by = Project.objects.filter(id=project_id, user_id=request.user.id).exists()
                project_follow_status = FollowedProject.objects.filter(project_id = project_id ,  user_id = request.user.id).exists()
                
                print(project_follow_status)
                return Response({"project_details": serializer.data,
                                 "save_status":save_status,
                                 "save_count":save_count,
                                 "project_follow_status":project_follow_status,
                                 "created_by":created_by
                                 },
                                status=200)
            except Project.DoesNotExist:
                return Response({"message": "Project not found for this user."}, status=404)
        return Response({"message": "Project ID is required."}, status=400)
    


class ProjectFeedbackListView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]

    def get(self, request, project_id):
        feedbacks = Feedback.objects.filter(project_id=project_id)
        if feedbacks.exists():
            serializer = FeedbackSerializer(feedbacks, many=True)
            return Response({"feedbacks": serializer.data}, status=200)
        return Response({"message": "No feedback found for this project."}, status=404)
    
    def post(self, request, project_id):
        try:
            project = Project.objects.get(id=project_id)
        except Project.DoesNotExist:
            return Response({"message": "Project not found."}, status=404)

        data = request.data.copy() 
        print(data) 
        data["project"] = project_id  # Attach the project ID to the request data
        data["user"] = request.user.id  # Attach the authenticated user's ID to the request data

        serializer = FeedbackSerializer(data=data)
        if serializer.is_valid():
            serializer.save()
            return Response({"message": "Feedback submitted successfully.", "feedback": serializer.data}, status=201)
        return Response({"errors": serializer.errors}, status=400)


class FeedbackDeleteView(APIView):

    def delete(self, request, project_id, feedback_id):
        # Get the project
        project = get_object_or_404(Project, id=project_id)

        # Check if the authenticated user is the creator of the project
        if project.user != request.user:
            return Response(
                {"error": "You are not authorized to delete feedback for this project."},
                status=status.HTTP_403_FORBIDDEN
            )

        # Get the specific feedback
        feedback = get_object_or_404(Feedback, id=feedback_id, project=project)

        # Delete the feedback
        feedback.delete()
        return Response({"message": "Feedback deleted successfully."}, status=status.HTTP_200_OK)


class AwardAPIView(APIView):

    def get(self, request, pk=None):
        """
        List all awards or retrieve a specific award by ID.
        """
        if pk:
            try:
                award = Award.objects.get(pk=pk)
                serializer = AwardSerializer(award)
                return Response(serializer.data, status=status.HTTP_200_OK)
            except Award.DoesNotExist:
                return Response({"error": "Award not found"}, status=status.HTTP_404_NOT_FOUND)
        else:
            awards = Award.objects.all()
            serializer = AwardSerializer(awards, many=True)
            return Response(serializer.data, status=status.HTTP_200_OK)

    def post(self, request):
        """
        Create a new award.
        """
        serializer = AwardSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def put(self, request, pk=None):
        """
        Update an existing award.
        """
        if not pk:
            return Response({"error": "Award ID is required for updating"}, status=status.HTTP_400_BAD_REQUEST)
        try:
            award = Award.objects.get(pk=pk)
        except Award.DoesNotExist:
            return Response({"error": "Award not found"}, status=status.HTTP_404_NOT_FOUND)
        
        serializer = AwardSerializer(award, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, pk=None):
        """
        Delete an award.
        """
        if not pk:
            return Response({"error": "Award ID is required for deletion"}, status=status.HTTP_400_BAD_REQUEST)
        try:
            award = Award.objects.get(pk=pk)
            award.delete()
            return Response({"message": "Award deleted successfully"}, status=status.HTTP_204_NO_CONTENT)
        except Award.DoesNotExist:
            return Response({"error": "Award not found"}, status=status.HTTP_404_NOT_FOUND)
        
class AwardByProjectID(APIView):
    permission_classes = [IsAuthenticated]

    def get(self,request, project_id):
        award = Award.objects.filter(project= project_id)
        if not award.exists():
            return Response({"error": "No awards found for the given project ID"}, status=status.HTTP_404_NOT_FOUND)

        serializer = AwardSerializer(award, many=True)  # Serialize queryset with many=True
        # return Response( {"Awards": serializer.data}, status=status.HTTP_200_OK)
        return Response({
            "status": "success",
            "message": "Awards retrieved successfully",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

class AddTeamView(APIView):
    permission_classes = [IsAuthenticated]  # Ensure the user is authenticated

    def post(self, request, project_id):
        # Retrieve the project and validate ownership
        project = get_object_or_404(Project, id=project_id)

        # Check if the requesting user is the owner of the project
        if project.user != request.user:
            return Response(
                {"error": "You are not authorized to add a team to this project."},
                status=status.HTTP_403_FORBIDDEN
            )

        # Validate and create the team
        data = request.data
        data['project'] = project.id  # Attach the project ID to the team data
        serializer = TeamSerializer(data=data, context={'request': request})
        if serializer.is_valid():
            serializer.save(created_by=request.user)  # Save the team with the creator
            return Response(
                {"message": "Team successfully added.", "team": serializer.data},
                status=status.HTTP_201_CREATED
            )

        # Handle validation errors
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

class RemoveTeamView(APIView):
    permission_classes = [IsAuthenticated]  # Ensure the user is authenticated

    def delete(self, request, team_id):
        # Retrieve the team object
        team = get_object_or_404(Team, id=team_id)

        # Check if the requesting user created the associated project
        if team.project.user != request.user:
            return Response(
                {"error": "You are not authorized to remove this team."},
                status=status.HTTP_403_FORBIDDEN
            )

        # Capture the team name for response before deletion
        team_name = team.team_name

        # Delete the team
        team.delete()

        return Response(
            {"message": f"Team '{team_name}' has been successfully removed."},
            status=status.HTTP_200_OK
        )

class AddTeamMemberView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, team_id):
        team = get_object_or_404(Team, id=team_id)
        project = team.project
        if project.user != request.user:
            return Response(
                {"error": "You are not authorized to add members to this team."},
                status=status.HTTP_403_FORBIDDEN
            )
        serializer = TeamMemberSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save(team=team)
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
class EditTeamMemberRoleView(APIView):
    permission_classes = [IsAuthenticated]  

    def patch(self, request, team_id, member_id):
        team = get_object_or_404(Team, id=team_id)
        project = team.project
        if project.user != request.user:
            return Response(
                {"error": "You are not authorized to edit members in this team."},
                status=status.HTTP_403_FORBIDDEN
            )
        member = get_object_or_404(TeamMember, id=member_id, team=team)
        role = request.data.get("role")
        if not role:
            return Response({"error": "Role is required."}, status=status.HTTP_400_BAD_REQUEST)

        member.role = role
        member.save()

        return Response(
            {"message": "Role updated successfully.", "name": member.name, "role": member.role},
            status=status.HTTP_200_OK
        )

class RemoveTeamMemberView(APIView):
    permission_classes = [IsAuthenticated]
    
    def delete(self, request, team_id, member_id):
        team = get_object_or_404(Team, id=team_id)
        project = team.project

        # Check if the requesting user created the project
        if project.user != request.user:
            return Response(
                {"error": "You are not authorized to remove members from this team."},
                status=status.HTTP_403_FORBIDDEN
            )

        member = get_object_or_404(TeamMember, id=member_id, team=team)
        member.delete()
        return Response({"message": "Team member removed successfully."}, status=status.HTTP_204_NO_CONTENT)  


class ProjectTeamMembersView(APIView):
    permission_classes = [IsAuthenticated]
    def get(self, request, project_id):
        try:
            # Fetch the project by ID
            project = Project.objects.get(id=project_id)

            # Retrieve teams and their members for the given project
            teams = Team.objects.filter(project=project).prefetch_related('members')

            # Construct the response
            response_data = {
                "project_id": project.id,
                "project_title": project.project_title,
                "teams": [
                    {
                        "team_name": team.team_name,
                        "members": [
                            {
                                "id": member.id,
                                "name": member.name,
                                "role": member.role
                            }
                            for member in team.members.all()
                        ],
                    }
                    for team in teams
                ],
            }

            return Response(response_data, status=200)

        except Project.DoesNotExist:
            raise NotFound({"message": "Project not found."})

# class EditProjectView(APIView):
#     permission_classes = [IsAuthenticated]
#     authentication_classes = [JWTAuthentication]
#     parser_classes = [MultiPartParser, FormParser]

#     def put(self, request, project_id):
#         try:
#             # Fetch the project by ID
#             project = Project.objects.get(id=project_id)

#             # Check if the authenticated user is the owner of the project
#             if project.user != request.user:
#                 raise PermissionDenied({"message": "You are not authorized to edit this project."})

#             # Update the project with the new data
#             serializer = ProjectUpdateSerializer(project, data=request.data, partial=True)
#             if serializer.is_valid():
#                 serializer.save()
#                 return Response({"message": "Project updated successfully.", "project": serializer.data}, status=status.HTTP_200_OK)

#             return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
#         except Project.DoesNotExist:
#             raise NotFound({"message": "Project not found."})

class EditProjectView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    parser_classes = [MultiPartParser, FormParser]

    def put(self, request, project_id):
        try:
            # Fetch the project by ID
            project = Project.objects.get(id=project_id)

            # Check if the authenticated user is the owner of the project
            if project.user != request.user:
                raise PermissionDenied({"message": "You are not authorized to edit this project."})

            # Update fields manually from the request data
            fields_to_update = [
                "project_title", "project_tagline", "character_name", "role_played",
                "project_status", "progress_percentage", "project_type", "language",
                "description", "primary_email", "secondary_email", "hashtags", "genre", "status"
            ]

            for field in fields_to_update:
                if field in request.data:
                    setattr(project, field, request.data[field])

            # Update media fields (images and files)
            if "press_release" in request.FILES:
                project.press_release = request.FILES["press_release"]
            if "media_release" in request.FILES:
                project.media_release = request.FILES["media_release"]
            if "thumbnail_image" in request.FILES:
                project.thumbnail_image = request.FILES["thumbnail_image"]
            if "cover_image" in request.FILES:
                project.cover_image = request.FILES["cover_image"]

            # Save the updated project
            project.save()

            return Response({"message": "Project updated successfully."}, status=200)

        except Project.DoesNotExist:
            raise NotFound({"message": "Project not found."})

        except ValidationError as e:
            return Response({"message": str(e)}, status=400)
    
class GlobalSearchView(APIView):
    ermission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    def get(self, request):
        username = request.query_params.get('username', None)  # Get username from query parameters

        if not username:
            return Response({"error": "Username is required"}, status=status.HTTP_400_BAD_REQUEST)

        # Query relevant models by username (assuming username is part of the related CustomUser model)
        casting_calls = CastingCall.objects.filter(user__username__icontains=username)
        internships = Internship.objects.filter(posted_by__username__icontains=username)
        event_registrations = EventRegistration.objects.filter(user__username__icontains=username)

        # Serialize the data
        casting_call_data = CastingCallSerializer(casting_calls, many=True).data
        internship_data = InternshipSerializer(internships, many=True).data
        event_registration_data = EventRegistrationSerializer(event_registrations, many=True).data

        # Combine all results into a single response
        response_data = {
            "casting_calls": casting_call_data,
            "internships": internship_data,
            "event_registrations": event_registration_data,
        }

        return Response(response_data, status=status.HTTP_200_OK)


class GigWorkFormView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]

    def post(self, request):
        serializer = GigWorkSerializer(data=request.data, context={'request': request})
        
        if serializer.is_valid():
            gig_work = serializer.save()  # Do NOT pass user here
           
            # Check if the project title is numeric and fetch accordingly
            project_title = gig_work.project_title
            if project_title.isdigit():
                # Convert to integer to find the nth project
                project_position = int(project_title)
                fetched_title = self.get_user_project_title(request.user, project_position)
                if fetched_title:
                    project_title = fetched_title  # Replace the numeric value with the actual title

            return Response({
                "status": {
                    "type": "success",
                    "message": "Gig work form submitted successfully",
                    "code": 200,
                    "error": False
                },
                "data": {
                    "form_id": gig_work.id,
                    "submission_status": "received",
                    "gig_title": gig_work.gig_title,
                    "user_id": gig_work.user.id,
                    "project_title": project_title,  # Show the resolved or custom title
                    # "document_url": gig_work.gigWork_document.url if gig_work.gigWork_document else None,
                }
            }, status=status.HTTP_200_OK)

        return Response({
            "status": {
                "type": "error",
                "message": "Validation Failed",
                "code": 400,
                "error": True
            },
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def get_user_project_title(self, user, position):
        """
        Get the nth project title for the authenticated user.
        """
        projects = Project.objects.filter(user=user).order_by('id')
        if 0 < position <= projects.count():
            return projects[position - 1].project_title
        return None

    
    def get(self, request, user_id=None):
        if user_id:
            # Filter gig works by user_id
            gig_works = GigWork.objects.filter(user_id=user_id)
            if gig_works.exists():  # Check if projects exist
                serializer = GigWorkSerializer(gig_works, many=True)
                return Response({
                    "data": serializer.data,
                    "user_id": user_id
                }, status=status.HTTP_200_OK)
            else:
                return Response({"message": "No gig works found for this user."}, status=status.HTTP_404_NOT_FOUND)
        
        # Return all gig works if no user_id is provided
        gig_works = GigWork.objects.all()
        serializer = GigWorkSerializer(gig_works, many=True)
        return Response({
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def put(self, request, user_id):
        try:
            gig_work = GigWork.objects.get(user_id=user_id)
            serializer = GigWorkSerializer(gig_work, data=request.data, partial=True)
            if serializer.is_valid():
                serializer.save()
                return Response({"message": "Gig work updated successfully."}, status=status.HTTP_200_OK)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        except GigWork.DoesNotExist:
            return Response({"message": "Gig work not found."}, status=status.HTTP_404_NOT_FOUND)

    def delete(self, request, user_id):
        try:
            gig_work = GigWork.objects.get(user_id=user_id)
            gig_work.delete()
            return Response({"message": "Gig work deleted successfully."}, status=status.HTTP_204_NO_CONTENT)
        except GigWork.DoesNotExist:
            return Response({"message": "Gig work not found."}, status=status.HTTP_404_NOT_FOUND)
        


class GigWorkSearchView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    
    def get(self, request):
        query = request.query_params.get('query', '')  # Search query
        filters = request.query_params  # Additional filters

        # Base queryset
        queryset = GigWork.objects.all()

        # Apply search query to relevant fields
        if query:
            queryset = queryset.filter(
                Q(gig_title__icontains=query) |
                Q(short_description__icontains=query) |
                Q(skills__icontains=query) |
                Q(project_title__icontains=query)
            )

        # Apply additional filters
        if 'gig_work_type' in filters:
            queryset = queryset.filter(gig_work_type=filters.get('gig_work_type'))
        if 'work_type' in filters:
            queryset = queryset.filter(work_type=filters.get('work_type'))
        if 'progress_of_project' in filters:
            queryset = queryset.filter(progress_of_project=filters.get('progress_of_project'))
        if 'work_method' in filters:
            queryset = queryset.filter(work_method=filters.get('work_method'))
        if 'promotion' in filters:
            queryset = queryset.filter(promotion=filters.get('promotion'))
        if 'min_price' in filters:
            queryset = queryset.filter(price__gte=filters.get('min_price'))
        if 'max_price' in filters:
            queryset = queryset.filter(price__lte=filters.get('max_price'))

        # Serialize the queryset
        serializer = GigWorkSerializer(queryset, many=True)

        # Return the response
        return Response(serializer.data, status=status.HTTP_200_OK)

# ----------------------------
# CastingCall View
# # ----------------------------   

class CastingCallFormEditView(APIView):
    permission_classes= [IsAuthenticated]
    authentication_classes=[JWTAuthentication]
    parser_classes=(MultiPartParser,FormParser)

    def put(self, request, casting_id):
            user = request.user

            # Check if the casting call exists and is associated with the current user
            try:
                casting_call = CastingCall.objects.get(user_id=user.id, id=casting_id)
            except CastingCall.DoesNotExist:
                return Response(
                    {"message": "Casting call not found."}, status=status.HTTP_404_NOT_FOUND
                )

            # Update the casting call with provided data
            serializer = CastingCallSerializer(casting_call, data=request.data, partial=True)
            if serializer.is_valid():
                serializer.save()
                return Response(
                    {"message": "Casting call updated successfully."},
                    status=status.HTTP_200_OK,
                )

            # Return validation errors
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
    def get(self,request,casting_id):
        user= request.user
        try:
            casting_call= CastingCall.objects.get(user_id=user.id,id=casting_id)
        except CastingCall.DoesNotExist:
            return Response(
                {"message":"Casting cal not found"},status=status.HTTP_404_NOT_FOUND
            )
        serializer = CastingCallSerializer(casting_call)
        return Response(
            {
                "success": True,
                "message": "Casting call retrieved successfully.",
                "data": serializer.data,
            },
            status=status.HTTP_200_OK,
        )
    
    def delete(self, request, casting_id):
        user= request.user
        try:
            casting_call = CastingCall.objects.get(user_id=user,id=casting_id)
            casting_call.delete()
            return Response({"message": "Casting call deleted successfully."}, status=status.HTTP_204_NO_CONTENT)
        except CastingCall.DoesNotExist:
            return Response({"message": "Casting call not found."}, status=status.HTTP_404_NOT_FOUND)
        

class CastingCallFormView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    parser_classes = (MultiPartParser, FormParser)

    def post(self, request):
        

        serializer = CastingCallSerializer(data=request.data, context={'request': request})  # Pass request context
        if serializer.is_valid():
            casting_call = serializer.save()
            return Response({
                "status": {
                    "type": "success",
                    "message": "Casting Call Created Successfully",
                    "code": 200,
                    "error": False
                },
                "data": {
                    "casting_call_id": casting_call.id,
                    "job_title": casting_call.job_title,
                    "project_title": casting_call.select_project,
                    "short_description": casting_call.short_description,
                    "created_at": casting_call.created_at,
                    "user_id": casting_call.user.id
                }
            }, status=status.HTTP_200_OK)

        return Response({
            "status": {
                "type": "error",
                "message": "Validation Failed",
                "code": 400,
                "error": True,
                "errors": serializer.errors
            }
        }, status=status.HTTP_400_BAD_REQUEST)
    
    def get(self, request, project_id):
            """
            Retrieve all casting calls for a specific project.
            """
            project = get_object_or_404(Project, id=project_id)
            casting_calls = project.casting_calls.all()
            serializer = CastingCallSerializer(casting_calls, many=True)
            # save_status = SavedCastingCall.objects.filter(internship_id = internship_id, user_id = user.id ).exists()
            # applied_status = InternshipApplication.objects.filter(internship = internship_id, user =user.id).exists()
            return Response({
                "status": "success",
                "message": "Casting calls retrieved successfully",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
    # def put(self, request, casting_id):
    #         user = request.user

    #         # Check if the casting call exists and is associated with the current user
    #         try:
    #             casting_call = CastingCall.objects.get(user_id=user.id, id=casting_id)
    #         except CastingCall.DoesNotExist:
    #             return Response(
    #                 {"message": "Casting call not found."}, status=status.HTTP_404_NOT_FOUND
    #             )

    #         # Update the casting call with provided data
    #         serializer = CastingCallSerializer(casting_call, data=request.data, partial=True)
    #         if serializer.is_valid():
    #             serializer.save()
    #             return Response(
    #                 {"message": "Casting call updated successfully."},
    #                 status=status.HTTP_200_OK,
    #             )

    #         # Return validation errors
    #         return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    # def delete(self, request, project_id):
    #     user= request.user
    #     try:
    #         casting_call = CastingCall.objects.get(user_id=user)
    #         casting_call.delete()
    #         return Response({"message": "Casting call deleted successfully."}, status=status.HTTP_204_NO_CONTENT)
    #     except CastingCall.DoesNotExist:
    #         return Response({"message": "Casting call not found."}, status=status.HTTP_404_NOT_FOUND)



class CastingCallSearchView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    def get(self, request):
        query = request.query_params.get('query', '')  # Get the search query from URL
        filters = request.query_params.get('filters', None)  # Optional additional filters

        # Base queryset
        queryset = CastingCall.objects.all()

        # Apply search query to relevant fields
        if query:
            queryset = queryset.filter(
                Q(job_title__icontains=query) |
                Q(short_description__icontains=query) |
                Q(skills__icontains=query) |
                Q(location__icontains=query)
            )


        # Example of additional filters
        if filters:
            filters = filters.split(',')  # Assuming filters is a comma-separated string
            if 'gender' in filters:
                gender = request.query_params.get('gender', None)
                if gender:
                    queryset = queryset.filter(gender=gender)
            if 'role_type' in filters:
                role_type = request.query_params.get('role_type', None)
                if role_type:
                    queryset = queryset.filter(role_type=role_type)

        serializer = CastingCallSerializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


class CastingCrewView(APIView):
    permission_classes = [IsAuthenticated]
    def post(self, request, project_id):
            try:
                # Validate project ownership
                project = Project.objects.get(id=project_id, user=request.user)

                # Deserialize data
                serializer = TeamSerializer(data=request.data)
                if serializer.is_valid():
                    team_name = serializer.validated_data['team_name']
                    team_members = serializer.validated_data['team_members']

                    # Create or get the team
                    team, created = Team.objects.get_or_create(
                        project=project, team_name=team_name
                    )

                    # Add members to the team
                    for member_data in team_members:
                        user = member_data.get('user')
                        name = member_data.get('name')
                        role = member_data.get('role')

                        # Validate user existence
                        if user:
                            try:
                                user_instance = CustomUser.objects.get(id=user.id)
                            except CustomUser.DoesNotExist:
                                return Response(
                                    {"error": f"User with ID {user.id} does not exist."},
                                    status=status.HTTP_404_NOT_FOUND,
                                )
                        else:
                            user_instance = None  # For null users

                        # Create the team member
                        TeamMember.objects.create(
                            team=team,
                            user=user_instance,
                            name=name,
                            role=role,
                        )

                    return Response(
                        {"message": "Team and members added successfully."},
                        status=status.HTTP_201_CREATED,
                    )

                return Response(
                    {"status": "error", "message": "Invalid data.", "errors": serializer.errors},
                    status=status.HTTP_400_BAD_REQUEST,
                )

            except Project.DoesNotExist:
                return Response(
                    {"error": "Project not found or you do not own this project."},
                    status=status.HTTP_404_NOT_FOUND,
                )
            
    def get(self, request, project_id):
        try:
            # Fetch the project
            project = Project.objects.get(id=project_id)
            
            # Fetch teams and prefetch team members
            teams = Team.objects.filter(project=project).prefetch_related("members")
            
            # Construct the response
            data = {
                "status": "success",
                "project_id": project.id,
                "teams": [
                    {
                        "team_id":team.id,
                        "team_name": team.team_name,
                        "team_members": [
                            {"name": member.name, "role": member.role}
                            for member in team.members.all()
                        ],
                    }
                    for team in teams
                ],
            }
            return Response(data, status=status.HTTP_200_OK)
        except Project.DoesNotExist:
            # Handle project not found
            return Response(
                {"status": "error", "message": "Project not found."},
                status=status.HTTP_404_NOT_FOUND,
            )

# class CastingCrewView(APIView):
#     permission_classes = [IsAuthenticated]

#     def post(self, request):
#         """
#         Handles a request from a user to join a team based on a casting call.
#         """
#         serializer = TeamJoinRequestSerializer(data=request.data)

#         # Validate serializer data
#         if serializer.is_valid():
#             user_id = serializer.validated_data["user_id"]
#             casting_call_id = serializer.validated_data["casting_call_id"]
#             team_id = serializer.validated_data["team_id"]

#             # Fetch user
#             if request.user.id != user_id:
#                 return Response(
#                     {"error": "You can only request to join teams using your own credentials."},
#                     status=status.HTTP_403_FORBIDDEN,
#                 )

#             # Validate casting call and team existence
#             user_instance = get_object_or_404(CustomUser, id=user_id)
#             casting_call_instance = get_object_or_404(CastingCall, id=casting_call_id)
#             team_instance = get_object_or_404(Team, id=team_id)

#             # Ensure the casting call belongs to the given team
#             if not casting_call_instance in team_instance.castingcall_set.all():
#                 return Response(
#                     {"error": "Casting call does not belong to this team."},
#                     status=status.HTTP_400_BAD_REQUEST,
#                 )

#             # Check if already requested
#             if CastingCallApplication.objects.filter(
#                 user=user_instance, team=team_instance, status="pending"
#             ).exists():
#                 return Response(
#                     {"error": "You have already submitted a request to this team."},
#                     status=status.HTTP_400_BAD_REQUEST,
#                 )

#             # Create a request record
#             CastingCallApplication.objects.create(
#                 user=user_instance, casting_call=casting_call_instance, team=team_instance
#             )

#             return Response(
#                 {"message": "Request submitted successfully."},
#                 status=status.HTTP_201_CREATED,
#             )

#         return Response(
#             {"error": "Invalid request data.", "errors": serializer.errors},
#             status=status.HTTP_400_BAD_REQUEST,
#         )


class CastingCallApplicationView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]
    def post(self, request, casting_call_id):
        user = request.user
        try:
            casting_call = CastingCall.objects.get(id=casting_call_id)
            print("hii",casting_call)

            # Check if user already applied
            if CastingCallApplication.objects.filter(user=user, casting_call=casting_call).exists():
                return Response({"message": "You have already applied for this casting call."}, status=status.HTTP_400_BAD_REQUEST)
            
            cover_letter = request.data.get("cover_letter")
            uploaded_file = request.FILES.get("uploaded_file")

            application = CastingCallApplication.objects.create(
                casting_call=casting_call,
                user=user,
                cover_letter=cover_letter,
                uploaded_file=uploaded_file,
                # team_id=team_id if team_id else None,
            )
            return Response(
                {
                    "message": "Application submitted successfully.",
                    "data": {
                        "application_id": application.id,
                        "status": application.status,
                        "applied_at": application.applied_at,
                    },
                },
                status=status.HTTP_201_CREATED,
            )
        except CastingCall.DoesNotExist:
            return Response({"message": "Casting call not found."}, status=status.HTTP_404_NOT_FOUND)

    def get(self, request, casting_call_id):
        user= request.user
        """
        List all applications for a specific casting call.
        """
        casting_call = get_object_or_404(CastingCall, id=casting_call_id)
        project_titile=casting_call.project.project_title if casting_call.project else "No project"

        print(project_titile)
        # Only the creator (team member) can view applications
        if request.user != casting_call.user:
            return Response(
                {"status": "error", "message": "You are not authorized to view applications."},
                status=status.HTTP_403_FORBIDDEN,
            )
        
        applications = CastingCallApplication.objects.filter(casting_call=casting_call,user=user)
        serializer = TeamJoinRequestSerializer(applications, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)
    
class UpdateCastingCallStatusView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, castingcall_id, *args, **kwargs):
        castingcall = get_object_or_404(CastingCall, pk=castingcall_id)
        serializer = CastingCallStatusSerializer(instance=castingcall, data=request.data, partial=True)

        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": "success",
                "message": "Casting call status updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "status": "error",
            "message": "Failed to update casting call status.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class ChangeProfileStatusView(APIView):
    VALID_STATUSES = ['selected', 'waiting_list', 'save_later', 'rejected']

    def post(self, request):
        userid = request.data.get('userid')
        profile_status = request.data.get('profile_status')
        casting_call_id = request.data.get('casting_call_id')

        # Check if profile_status is valid
        if profile_status not in self.VALID_STATUSES:
            return Response({
                "status": "error",
                "message": "Invalid profile status. Valid statuses are 1 to 5."
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            casting_call = CastingCall.objects.get(id=casting_call_id, user=request.user)
            application = CastingCallApplication.objects.get(user_id=userid, casting_call=casting_call)
            application.status = profile_status
            application.save()

            Notification.objects.create(
                user=application.user,  # Notify the applicant
                action_type="applicant_status",
                message=f"Your application for the Casting call has been updated. Current status: {application.status}."
            )
            if application.user.privacy_settings.enable_notifications:
                response = send_push_notification(
                    fcm_token=application.user.fcm_token,
                    title="Application status",
                    body=f"Your application for the Casting call has been updated. Current status: {application.status}."
                )
            else:
                response= None

            # Send notification based on the profile status
            # notification_message = self.get_notification_message(profile_status)
            # self.send_notification(user, notification_message)

            return Response({
                "status": "success",
                "push_notification": response,
                "message": "Profile status updated successfully",
                # "notification": notification_message
            }, status=status.HTTP_200_OK)

        except CastingCall.DoesNotExist:
            return Response({
                "status": "error",
                "message": "Casting call not found or you are not authorized to update it."
            }, status=status.HTTP_403_FORBIDDEN)
        except CastingCallApplication.DoesNotExist:
            return Response({
                "status": "error",
                "message": "Casting call application not found for the user."
            }, status=status.HTTP_404_NOT_FOUND)


class AppliedUsersView(APIView):
    permission_classes = [IsAuthenticated]
    authentication_classes = [JWTAuthentication]

    def get(self, request, casting_call_id):
        # Get the casting call
        casting_call = get_object_or_404(CastingCall, id=casting_call_id)
        print(casting_call)
        # Get all users who applied
        applications = casting_call.applications.select_related('user')  # Optimize query with select_related
        users = [application.user for application in applications]
        print(users)

        # Serialize user data
        serializer = CustomUserSerializer(
            [application.user for application in applications],
            many=True
        )
        # print(serializer.data)
        serializer_data = serializer.data
        for i in range(len(serializer_data)):
            user_data = applications[i].user.profile.location
            parts = user_data.split(',')
            if len(parts) == 2:
                city_id, state_id = map(int, parts)  # Convert both parts to integers

                # Fetch city and state names
                city = City.objects.filter(id=city_id).first()
                state = State.objects.filter(id=state_id).first()

                if city and state:
                    location = f"{city.name}, {state.name}"
                else:
                    location = None

            serializer_data[i]["location"]= location
            primary_industry = applications[i].user.profile.selected_primary_industry.name
            serializer_data[i]["primary_industry"]= primary_industry
            primary_skill = applications[i].user.profile.selected_primary_skill.name
            serializer_data[i]["primary_skill"]= primary_skill

            serializer_data[i]["cover_letter"]= applications[i].cover_letter
            serializer_data[i]["uploaded_file"]= applications[i].uploaded_file.url
            serializer_data[i]["applied_at"]= applications[i].applied_at
            serializer_data[i]["status"]= applications[i].status

            # serializer_data[i]["application"] = {
            #     "cover_letter": application.cover_letter,
            #     "uploaded_file": request.build_absolute_uri(application.uploaded_file.url) if application.uploaded_file else None,
            #     "applied_at": application.applied_at.strftime('%Y-%m-%d %H:%M:%S'),
            #     "status": application.status
            # }


        return Response({
            "success": True,
            "message": "Applied users retrieved successfully.",
            "data": serializer.data
        }, status=status.HTTP_200_OK)

# ----------------------------
# Project View
# ----------------------------
class ProjectFormView(APIView):
    permission_classes = [IsAuthenticated]  
    authentication_classes = [JWTAuthentication]

    def post(self, request):
        mode = request.data.get("mode", "default").lower()
        project_id = request.data.get("project_id")

        try:
            if project_id:
                project = Project.objects.get(id=project_id, user=request.user)
            else:
                project = None
        except Project.DoesNotExist:
            return Response({"error": "Project not found."}, status=status.HTTP_404_NOT_FOUND)
        # print(request.data.get("cover_image_url"))
        serializer = CreateProjectSerializer(instance=project, data=request.data, context={'request': request})
        if serializer.is_valid():
            project = serializer.save()
            press_release_files = request.FILES.getlist('press_releases')
            for file in press_release_files:
                PressRelease.objects.create(project=project, image=file)

            # Handle multiple media release files (images/videos)
            media_files = request.FILES.getlist('media_releases')
            for file in media_files:
                MediaRelease.objects.create(project=project, file=file)

            return Response({'success': True, 'data': serializer.data}, status=201)
        return Response({'success': False, 'errors': serializer.errors}, status=400)

        #     # if mode == "publish":
        #     try:
        #         project.validate_publish()  # Validate mandatory fields for publishing
        #         project.project_status = "Published"
        #         project.save()
        #     except ValidationError as e:
        #         return Response({"error": e.message_dict}, status=status.HTTP_400_BAD_REQUEST)

        #     # elif mode == "draft":
        #     #     project.project_status = "Draft"
        #     #     project.save()
        #     return Response({'success': True, "data": ProjectSerializer(project).data}, status=status.HTTP_201_CREATED)
        # else:
        #     raise ValidationError(serializer.errors)
    
    def get(self, request, user_id=None):
            if user_id:
                # Use `filter()` instead of `get()` to return all projects for a user
                projects = Project.objects.filter(user_id=user_id)
                if projects.exists():  # Check if any projects are found
                    serializer = ProjectSerializer(projects, many=True, context={'request': request})
                    return Response({"data": serializer.data}, status=status.HTTP_200_OK)
                else:
                    return Response({"message": "No projects found for this user."}, status=status.HTTP_404_NOT_FOUND)
            
            # If no `user_id` is provided, return all projects
            projects = Project.objects.all()
            serializer = ProjectSerializer(projects, many=True)

            return Response({"data": serializer.data}, status=status.HTTP_200_OK)
    def put(self, request, user_id):
        try:
            project = Project.objects.get(user_id=user_id)
            serializer = ProjectSerializer(project, data=request.data, partial=True)
            if serializer.is_valid():
                serializer.save()
                return Response({"message": "Project updated successfully."}, status=status.HTTP_200_OK)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        except Project.DoesNotExist:
            return Response({"message": "Project not found."}, status=status.HTTP_404_NOT_FOUND)

    def delete(self, request, user_id):
        try:
            project = Project.objects.get(user_id=user_id)
            project.delete()
            return Response({"message": "Project deleted successfully."}, status=status.HTTP_204_NO_CONTENT)
        except Project.DoesNotExist:
            return Response({"message": "Project not found."}, status=status.HTTP_404_NOT_FOUND)


class ProjectCastingCallListView(APIView):
    permission_classes = [IsAuthenticated]  
    authentication_classes = [JWTAuthentication]
    
    def get(self, request, project_id):
        try:
            # Get the project by its ID
            project = Project.objects.get(id=project_id)
            
            # Fetch all casting calls related to this project
            casting_calls = CastingCall.objects.filter(project=project)
            print(casting_calls)
            # Serialize the casting call data
            serializer = CastingCallSerializer(casting_calls, many=True)
            
            return Response({
                "status": {
                    "type": "success",
                    "message": "Casting calls retrieved successfully",
                    "code": 200,
                    "error": False
                },
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        
        except Project.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "message": "Project not found",
                    "code": 404,
                    "error": True
                }
            }, status=status.HTTP_404_NOT_FOUND)

# ----------------------------
# Event Registration View
# ----------------------------


class EventRegistrationCreateView(APIView):

    permission_classes = [IsAuthenticated]  
    authentication_classes = [JWTAuthentication]
    @swagger_auto_schema(request_body=EventRegistrationSerializer)
    # parser_classes = (MultiPartParser, FormParser)
    def post(self, request):
        serializer = EventRegistrationSerializer(data=request.data, context={'request': request})

        if serializer.is_valid():
            event_registration = serializer.save()  # Calls the create method of the serializer
            return Response({
                "status": {
                    "type": "success",
                    "message": "Event registration submitted successfully",
                    "code": 201,
                    "error": False
                },
                "data": {
                    "user_id":event_registration.user_id,
                    "event_id": event_registration.id,
                    "event_status": "Pending approval",
                    "submission_status": "Submitted"
                }
            }, status=status.HTTP_201_CREATED)

        return Response({
            "status": {
                "type": "error",
                "message": "Invalid data",
                "code": 400,
                "error": True,
                "errors": serializer.errors
            }
        }, status=status.HTTP_400_BAD_REQUEST)

    def get(self, request, user_id=None):
        if user_id:
            try:
                # Fetch the EventRegistration for the provided user_id
                event_registration = EventRegistration.objects.get(user_id=user_id)
                serializer = EventRegistrationSerializer(event_registration)
                return Response({
                    "data": serializer.data,
                    "user_id": user_id
                }, status=status.HTTP_200_OK)
            except EventRegistration.DoesNotExist:
                return Response({"message": "Event registration not found for this user."}, status=status.HTTP_404_NOT_FOUND)

        # Fetch all event registrations if no user_id is provided
        event_registrations = EventRegistration.objects.all()
        serializer = EventRegistrationSerializer(event_registrations, many=True)
        return Response({"data": serializer.data}, status=status.HTTP_200_OK)

    def put(self, request, user_id):
        try:
            event_registration = EventRegistration.objects.get(user_id=user_id)
            serializer = EventRegistrationSerializer(event_registration, data=request.data, partial=True)
            if serializer.is_valid():
                serializer.save()
                return Response({"message": "Event registration updated successfully."}, status=status.HTTP_200_OK)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        except EventRegistration.DoesNotExist:
            return Response({"message": "Event registration not found."}, status=status.HTTP_404_NOT_FOUND)

    def delete(self, request, user_id):
        try:
            event_registration = EventRegistration.objects.get(user_id=user_id)
            event_registration.delete()
            return Response({"message": "Event registration deleted successfully."}, status=status.HTTP_204_NO_CONTENT)
        except EventRegistration.DoesNotExist:
            return Response({"message": "Event registration not found."}, status=status.HTTP_404_NOT_FOUND)

    

# class CastingCallFormView(APIView):
#     permission_classes = [IsAuthenticated]
#     authentication_classes = [JWTAuthentication]
#     parser_classes = (MultiPartParser, FormParser)
#     def get(self, request, project_id):
#         """
#         Retrieve all casting calls for a specific project.
#         """
#         project = get_object_or_404(Project, id=project_id)
#         casting_calls = project.casting_calls.all()
#         serializer = CastingCallSerializer(casting_calls, many=True)
#         return Response(serializer.data, status=status.HTTP_200_OK)

class OpportunitiesGlobalListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        # Fetch opportunities from each model
        casting_calls = CastingCall.objects.all()
        gig_works = GigWork.objects.all()
        
        apprenticeships = Apprenticeship.objects.all()
        internships = Internship.objects.all()

        # Serialize the data
        casting_calls_data = CastingCallSerializer(casting_calls, many=True, context={'request': request}).data
        gig_works_data = GigWorkSerializer(gig_works, many=True, context={'request': request}).data
        apprenticeships_data = ApprenticeshipSerializer(apprenticeships, many=True, context={'request': request}).data
        internships_data = InternshipSerializer(internships, many=True, context={'request': request}).data

        all_opportunities = []

        for item in casting_calls_data:
            item['type'] = 'Casting Call'
            all_opportunities.append(item)

        for item in gig_works_data:
            item['type'] = 'Gig Work'
            all_opportunities.append(item)

        for item in apprenticeships_data:
            item['type'] = 'Apprenticeship'
            all_opportunities.append(item)

        for item in internships_data:
            item['type'] = 'Internship'
            all_opportunities.append(item)
            
        sorted_opportunities = sorted(
        all_opportunities,
        key=lambda x: datetime.fromisoformat(x['created_at'].replace('Z', '+00:00')),
        reverse=True
    )
        
        page = request.GET.get('page', 1)  # Get page number from request, default to 1
        page_size = request.GET.get('page_size', 10)  # Get page size, default to 10

        paginator = Paginator(sorted_opportunities, page_size)
        paginated_data = paginator.get_page(page)

        return Response({
            "data": list(paginated_data),  # Convert Page object to list
            "current_page": paginated_data.number,
            "total_pages": paginator.num_pages,
            "total_items": paginator.count,
            "status": "success",
            "message": "Fetched successfully"
        })

#         return Response({
#             "data": sorted_opportunities,
#             "status": "success",
#             "message": "Fetched successfully"
#         }
# )

class SearchBarView(APIView):
    def post(self, request):
        payload = request.data
        keyword = payload.get("keyword", "").strip()
        particular_keys = payload.get("particular_key", [])

        if not keyword:
            return Response({"error": "Keyword is required"}, status=400)

        # Initialize result sets
        internships = []
        projects = []
        gigworks = []
        casting_calls = []
        apprenticeships = []
        events = []
        users = []

        if not particular_keys or "internship" in particular_keys:
            internships = Internship.objects.filter(
                Q(fullname__icontains=keyword) |
                Q(organisation_or_individual__icontains=keyword) |
                Q(profile_title__icontains=keyword) |
                # Q(skills_required__icontains=keyword) |
                Q(job_responsibilities__icontains=keyword)
            )

        if not particular_keys or "Project" in particular_keys:
            projects = Project.objects.filter(
                Q(project_title__icontains=keyword) |
                Q(project_tagline__icontains=keyword) |
                Q(character_name__icontains=keyword) |
                Q(role_played__icontains=keyword) |
                Q(description__icontains=keyword) |
                Q(hashtags__icontains=keyword)
            )
        if not particular_keys or "gigworks" in particular_keys:
            gigworks = GigWork.objects.filter(
                Q(title__icontains=keyword) |
                Q(short_description__icontains=keyword) |
                Q(work_hours__icontains=keyword) |
                Q(work_type__icontains=keyword) |
                Q(salary__icontains=keyword) |
                Q(skills__icontains=keyword)
            )
        if not particular_keys or "Castingcall" in particular_keys:
            casting_calls = CastingCall.objects.filter(
                Q(job_title__icontains=keyword) |
                Q(select_project__icontains=keyword) |
                Q(short_description__icontains=keyword) |
                Q(gender__icontains=keyword) |
                Q(experience__icontains=keyword) |
                Q(role_type__icontains=keyword) |
                Q(skills__icontains=keyword)
            )

        if not particular_keys or "apprenticeship" in particular_keys:
            apprenticeships = Apprenticeship.objects.filter(
                Q(fullname__icontains=keyword) |
                Q(organisation_or_individual__icontains=keyword) |
                # Q(skills_required__icontains=keyword) |
                Q(job_responsibilities__icontains=keyword) |
                Q(start_date__icontains=keyword) |
                Q(end_date__icontains=keyword)
            )

        if not particular_keys or "Events" in particular_keys:
            events = Event.objects.filter(
                Q(title__icontains=keyword) |
                Q(overview__icontains=keyword) |
                Q(location__icontains=keyword) |
                Q(event_category__icontains=keyword)
            )

        if not particular_keys or "Artizt" in particular_keys:
            users = cuser.objects.filter(
                Q(email__icontains=keyword) |
                Q(full_name__icontains=keyword) |
                Q(username__icontains=keyword)
            )
        

        # result_data = {}
        # if not particular_keys or "Artizt" in particular_keys:
        #     users = cuser.objects.filter(
        #         Q(email__icontains=keyword) |
        #         Q(full_name__icontains=keyword) |
        #         Q(username__icontains=keyword)
        #     ).values()
        # result_data['Users'] = users

        internship_data = InternshipViewSerializer(internships, many=True).data
        project_data = ProjectSerializer(projects, many=True, context={'request': request}).data
        gigwork_data = GigWorkSerializer(gigworks, many=True, context={'request': request}).data
        casting_call_data = CastingCallSerializer(casting_calls, many=True).data
        apprenticeship_data = ApprenticeshipViewSerializer(apprenticeships, many=True).data
        event_data = EventSerializer(events, many=True).data
        users_data = CustomUserSerializer(users, many=True).data
        
        return Response({
            "Internships": internship_data,
            "Projects": project_data,
            "Gigworks": gigwork_data,
            "CastingCalls": casting_call_data,
            "Apprenticeships": apprenticeship_data,
            "Events": event_data,
            "Users": users_data
        })

class SearchView(APIView):
    def get(self, request):
        keyword = request.query_params.get("keyword", "").strip()
        sortname = request.query_params.get("sortname", "").strip().lower()

        if not keyword:
            return Response({"error": "Keyword is required"}, status=400)

        if not sortname:
            return Response({"error": "Sortname is required"}, status=400)

        # Map sortname to model and serializer
        sortname_mapping = {
            "internship": (Internship, InternshipViewSerializer),
            "project": (Project, ProjectSerializer),
            "gigwork": (GigWork, GigWorkSerializer),
            "castingcall": (CastingCall, CastingCallSerializer),
            "apprenticeship": (Apprenticeship, ApprenticeshipViewSerializer),
            "event": (Event, EventSerializer),
        }

        # Check if the sortname is valid
        if sortname not in sortname_mapping:
            return Response({"error": f"Invalid sortname: {sortname}"}, status=400)

        # Get the model and serializer
        model, serializer_class = sortname_mapping[sortname]


        # General model search
        filter_fields = {
            "internship": ["fullname", "organisation_or_individual", "profile_title", "job_responsibilities"],
            "project": ["project_title", "project_tagline", "character_name", "role_played", "description", "hashtags"],
            "gigwork": ["title", "short_description", "work_hours", "work_type", "salary", "skills"],
            "castingcall": ["job_title", "select_project", "short_description", "gender", "experience", "role_type", "skills"],
            "apprenticeship": ["fullname", "organisation_or_individual", "profile_title", "job_responsibilities"],
            "event": ["title", "overview", "location", "event_category"],
        }


        query = Q()
        for field in filter_fields.get(sortname, []):
            query |= Q(**{f"{field}__icontains": keyword})

        results = model.objects.filter(query)

        # Serialize results
        if serializer_class:
            serialized_data = serializer_class(results, many=True).data
            return Response({
                "success": True,
                "message": f"Results for {sortname.capitalize()} search.",
                "count": len(serialized_data),
                "data": serialized_data
            })
        else:
            return Response({"error": "No serializer defined for this sortname"}, status=400)

    

class GetSortlist(APIView):
    permission_classes = [IsAuthenticated]
    
    def get(self, request):
        return Response({
            'success' : True,
            'data' : [
                        {"id": 1, "sortname": "Artizt"},
                        {"id": 2, "sortname": "Castingcall"},
                        {"id": 3, "sortname": "Project"},
                        {"id": 4, "sortname": "Events"},
                        {"id": 5, "sortname": "gigworks"},
                        {"id": 6, "sortname": "internship"},
                        {"id": 7, "sortname": "apprenticeship"}
                    ]
        }, status = status.HTTP_200_OK)


## apprenticeship management

# class CreateApprenticeshipView(APIView):
#     permission_classes = [IsAuthenticated]

#     def post(self, request):
#         data = request.data.copy()
#         data['posted_by'] = request.user.id
#         skills = data.get('skills_required', None)
        
#         if skills is not None:
#             if isinstance(skills, str):
#                 try:
#                     skills = json.loads(skills)
#                 except json.JSONDecodeError:
#                     raise ValidationError({"skill": "Expected a list of skills IDs."})
#             data['skills_required'] = skills

#         serializer = ApprenticeshipSerializer(data=data)

#         if serializer.is_valid():
#             serializer.save()

#             return Response(
#                 {"success": True, "message": "Apprenticeship created successfully."},
#                 status=status.HTTP_201_CREATED,
#             )
#         raise ValidationError(serializer.errors)
class CreateApprenticeshipView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        data = request.data

        # Extract fields from request data
        fullname = data.get('fullname')
        organisation_or_individual = data.get('organisation_or_individual')
        profile_title = data.get('profile_title')
        email = data.get('email')
        designation = data.get('designation')
        job_responsibilities = data.get('job_responsibilities')
        location = data.get('location')
        apprenticeship_type = data.get('apprenticeship_type')
        start_date = data.get('start_date')
        end_date = data.get('end_date')
        duration = data.get('duration')
        salary = data.get('salary')
        gender = data.get('gender', 'Any')
        education_required = data.get('education_required')
        age_required = data.get('age_required')
        skills_required = data.get('skills_required', '')
        skills_experience = data.get('skills_experience', '')
        is_active = data.get('is_active', True)
        status_field = data.get('status', 'opening')
        logo = request.FILES.get('logo')
        thumbnail = request.FILES.get('thumbnail')
        language = data.get('language')
        mobile_number = data.get('mobile_number')

        # Validate skills_required and skills_experience
        print(skills_required)
        skills_required = skills_required.split(',') if isinstance(skills_required, str) and skills_required else []
        skills_experience = skills_experience.split(',') if isinstance(skills_experience, str) and skills_experience else []
        

        try:
            skills_required = list(map(int, skills_required))
            print(skills_required)
            skills_experience = list(map(str, skills_experience))
            print(skills_experience)
        except ValueError:
            return Response({"error": "skills_required and skills_experience must contain valid integers."}, status=status.HTTP_400_BAD_REQUEST)
        
        if len(skills_required) != len(skills_experience):
            return Response(
                {"error": "skills_required and skills_experience must have the same length."}, 
                status=status.HTTP_400_BAD_REQUEST
            )
        
        # Validate skills IDs
        skills_objects = []
        for skill_id in skills_required:
            skill = get_object_or_404(Skill, id=skill_id)
            skills_objects.append(skill)

        # Create Apprenticeship instance
        apprenticeship = Apprenticeship.objects.create(
            fullname=fullname,
            organisation_or_individual=organisation_or_individual,
            profile_title=profile_title,
            email=email,
            job_responsibilities=job_responsibilities,
            designation=designation,
            location=location,
            apprenticeship_type=apprenticeship_type,
            start_date=start_date,
            end_date=end_date,
            duration=duration,
            salary=salary,
            gender=gender,
            education_required=education_required,
            age_required=age_required,
            skills_experience=skills_experience,
            posted_by=request.user,
            is_active=is_active,
            status=status_field,
            logo=logo,
            thumbnail=thumbnail,
            language=language,
            mobile_number= mobile_number,
        )

        # Add skills_required to the apprenticeship
        apprenticeship.skills_required.set(skills_objects)
        apprenticeship.save()

        return Response(
            {"success": True, "message": "Apprenticeship created successfully."},
            status=status.HTTP_201_CREATED
        )
    
class ApprenticeshipDetailView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, apprenticeship_id):
        user = request.user
        try:
            apprenticeship = Apprenticeship.objects.get(id=apprenticeship_id, is_active=True)
        except Apprenticeship.DoesNotExist:
            return Response(
                {"success": False, "message": "Apprenticeship not found."},
                status=status.HTTP_404_NOT_FOUND,
            )

        serializer = ApprenticeshipViewSerializer(apprenticeship)
        create_by = apprenticeship.posted_by
        create_by_status = create_by.id == user.id

        save_status = SavedApprenticeship.objects.filter(apprenticeship_id = apprenticeship_id, user_id = user.id ).exists()
        applied_status = ApprenticeApplication.objects.filter(apprenticeship = apprenticeship_id, user =user.id).exists()
        save_count = SavedApprenticeship.objects.filter(user_id = user.id ).count()
        return Response(
            {"success": True, "data": serializer.data, "create_by":create_by_status, "save_status": save_status,
             "applied_status": applied_status,
              "saved_count":save_count,
                },
            status=status.HTTP_200_OK,
        )

class EditApprenticeshipView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, apprenticeship_id):
        try:
            apprenticeship = get_object_or_404(Apprenticeship, id=apprenticeship_id, posted_by=request.user)
            # apprenticeship = Apprenticeship.objects.get(id=apprenticeship_id, posted_by=request.user)
        except Apprenticeship.DoesNotExist:
            return Response(
                {"success": False, "message": "Apprenticeship not found or unauthorized action."},
                status=status.HTTP_404_NOT_FOUND,
            )


        serializer = ApprenticeshipEditSerializer(apprenticeship, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()

            return Response(
                {"success": True, "message": "Apprenticeship updated successfully."},
                status=status.HTTP_200_OK,
            )

        # Return validation errors
        return Response(
            {"success": False, "message": "Invalid data.", "errors": serializer.errors},
            status=status.HTTP_400_BAD_REQUEST,
        )

class MyApprenticeshipListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        # apprenticeships = Apprenticeship.objects.filter(posted_by=user, is_active=True).order_by('-created_at')
        apprenticeships = (Apprenticeship.objects.filter(posted_by=user, is_active=True)
        .annotate(applications_count=Count('applications'))
        .order_by('-created_at')
        )

        serializer = ApprenticeshipViewSerializer(apprenticeships, many=True)
        if serializer.data:
            return Response({
                "success": True,
                "message": "Apprenticeship list fetched successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                "success": False,
                "message": "data not found"
            }, status=status.HTTP_404_NOT_FOUND)

class AppliedApprenticeshipListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        # Fetch applications by the user
        applications = ApprenticeApplication.objects.filter(user=user).select_related('apprenticeship').order_by('-applied_at')

        serializer = AppliedApprenticeshipSerializer(applications, many=True, context={'request': request})
        if serializer.data:
            return Response({
                "success": True,
                "message": "Applied apprenticeship list fetched successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                "success": False,
                "message": "data not found"
            }, status=status.HTTP_404_NOT_FOUND)

class UpdateApprenticeshipApplicationStatusView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, apprenticeship_id):
        data = request.data
        user_id = data.get('user_id', None)

        try:
            application = ApprenticeApplication.objects.get(user=user_id, apprenticeship=apprenticeship_id)
        except ApprenticeApplication.DoesNotExist:
            return Response({
                "success": False,
                "message": "Apprenticeship application not found."
            }, status=status.HTTP_404_NOT_FOUND)

        # Serialize and validate the input
        serializer = ApprenticeshipApplicationStatusUpdateSerializer(application, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            Notification.objects.create(
                user=application.user,  # Notify the applicant
                action_type="applicant_status",
                message=f"Your application for the apprenticeship '{application.apprenticeship.fullname}' has been updated. Current status: {application.status}."
            )
            recipient_user = get_object_or_404(CustomUser, id=user_id)
            privacy_settings, created = PrivacySettings.objects.get_or_create(user=recipient_user)
            # if application.user.privacy_settings.enable_notifications:
            if privacy_settings.enable_notifications:
                response = send_push_notification(
                    fcm_token=application.user.fcm_token,
                    title="Application status",
                    body=f"Your application for the apprenticeship '{application.apprenticeship.fullname}' has been updated. Current status: {application.status}."
                )
            else:
                response = None
            return Response({
                "success": True,
                # "push_notification": response,
                "message": "Apprenticeship application status updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "success": False,
            "message": "Invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class UpdateApprenticeshipStatusView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, apprenticeship_id, *args, **kwargs):
        apprenticeship = get_object_or_404(Apprenticeship, pk=apprenticeship_id, posted_by=request.user)
        serializer = ApprenticeshipStatusSerializer(instance=apprenticeship, data=request.data, partial=True)

        if serializer.is_valid():
            serializer.save()
            return Response({
                "success": "true",
                "message": "Apprenticeship status updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "status": "error",
            "message": "Failed to update apprenticeship status.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class ApplyForApprenticeshipView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        serializer = ApprenticeshipApplicationSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            # Save the application with the current user
            serializer.save(user=request.user)
            return Response({
                "success": True,
                "message": "You have successfully applied for the apprenticeship.",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)

        return Response({
            "success": False,
            "message": "Invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class BaseApplicationListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, apprenticeship_id, status_filter):
        try:
            apprenticeship = Apprenticeship.objects.get(id=apprenticeship_id, posted_by=request.user)
        except Apprenticeship.DoesNotExist:
            return Response({
                "success": False,
                "message": "Apprenticeship not found or you are not authorized to view applications."
            }, status=status.HTTP_404_NOT_FOUND)
        
        age_limit = request.GET.get('age', None)

        # Filter applications by status
        applications = ApprenticeApplication.objects.filter(apprenticeship=apprenticeship, status=status_filter)
        if age_limit not in [None, '']:
            try:
                age_limit = int(age_limit)  # Convert age_limit to an integer
                applications = applications.filter(user__profile__age__lte=age_limit)  # Assuming `age` is in the `Profile` model
            except ValueError:
                return Response({
                    "success": False,
                    "message": "Invalid age parameter. Please provide a valid integer value."
                }, status=status.HTTP_400_BAD_REQUEST)

        # Serialize and return the data
        serializer = UserApplicationSerializer(applications, many=True, context={'request': request})
        serializer_data= serializer.data
        for i in range(len(serializer_data)):
            user_data = applications[i].user.profile.location
            print(user_data)
            location = "Unknown Location"
            if user_data:
                parts = user_data.split(',')
                if len(parts) == 2 and parts[0].strip().isdigit() and parts[1].strip().isdigit():
                    city_id, state_id = map(int, parts)  # Convert both parts to integers

                    # Fetch city and state names
                    city = City.objects.filter(id=city_id).first()
                    state = State.objects.filter(id=state_id).first()

                    if city and state:
                        location = f"{city.name}, {state.name}"

            serializer_data[i]["location"]= location
            primary_industry = applications[i].user.profile.selected_primary_industry.name
            serializer_data[i]["primary_industry"]= primary_industry
            primary_skill = applications[i].user.profile.selected_primary_skill.name
            serializer_data[i]["primary_skill"]= primary_skill
            age = applications[i].user.profile.age
            serializer_data[i]["age"]= age
            # description
            description = applications[i].description
            serializer_data[i]["description"]= description
            user_status = applications[i].status
            serializer_data[i]["status"]= user_status
        if serializer.data:
            return Response({
                "success": True,
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                "success": False,
                "message": "data not found"
            }, status=status.HTTP_404_NOT_FOUND)

class AppliedUserListView(BaseApplicationListView):
    def get(self, request, apprenticeship_id):
        return super().get(request, apprenticeship_id, status_filter='applied')

class SelectedUserListView(BaseApplicationListView):
    def get(self, request, apprenticeship_id):
        return super().get(request, apprenticeship_id, status_filter='selected')


class WaitingListUserListView(BaseApplicationListView):
    def get(self, request, apprenticeship_id):
        return super().get(request, apprenticeship_id, status_filter='waiting')

class SaveLaterUserListView(BaseApplicationListView):
    def get(self, request, apprenticeship_id):
        return super().get(request, apprenticeship_id, status_filter='saved')

class RejectedUserListView(BaseApplicationListView):
    def get(self, request, apprenticeship_id):
        return super().get(request, apprenticeship_id, status_filter='rejected')
    


## Internship Management

# class CreateInternshipView(APIView):
#     permission_classes = [IsAuthenticated]

#     def post(self, request):
#         data = request.data.copy()
#         data['posted_by'] = request.user.id
#         skills = data.get('skills_required', None)
        
#         if skills is not None:
#             if isinstance(skills, str):
#                 try:
#                     skills = json.loads(skills)
#                 except json.JSONDecodeError:
#                     raise ValidationError({"skill": "Expected a list of skills IDs."})
#             data['skills_required'] = skills

#         serializer = InternshipSerializer(data=data)

#         if serializer.is_valid():
#             serializer.save()

#             return Response(
#                 {"success": True, "message": "internship created successfully."},
#                 status=status.HTTP_201_CREATED,
#             )
#         raise ValidationError(serializer.errors)

class CreateInternshipView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        data = request.data
        fullname = data.get('fullname')
        organisation_or_individual = data.get('organisation_or_individual')
        profile_title = data.get('profile_title')
        email = data.get('email')
        job_responsibilities = data.get('job_responsibilities')
        designation = data.get('designation')
        location = data.get('location')
        internship_type = data.get('internship_type')
        start_date = data.get('start_date')
        end_date = data.get('end_date')
        duration = data.get('duration')
        salary = data.get('salary')
        gender = data.get('gender', 'Male')
        education_required = data.get('education_required')
        age_required = data.get('age_required')
        skills_required = data.get('skills_required', '')
        skills_experience = data.get('skills_experience', '')
        is_active = data.get('is_active', True)
        status_field = data.get('status', 'opening')
        thumbnail = request.FILES.get('thumbnail')
        logo = request.FILES.get('logo')
        language = data.get('language')
        mobile_number = data.get('mobile_number')

        print(skills_required)
        skills_required = skills_required.split(',') if isinstance(skills_required, str) and skills_required else []
        skills_experience = skills_experience.split(',') if isinstance(skills_experience, str) and skills_experience else []
        

        try:
            skills_required = list(map(int, skills_required))
            print(skills_required)
            skills_experience = list(map(str, skills_experience))
            print(skills_experience)
        except ValueError:
            return Response({"error": "skills_required and skills_experience must contain valid integers."}, status=status.HTTP_400_BAD_REQUEST)

        if len(skills_required) != len(skills_experience):
            return Response({"error": "skills_required and skills_experience must have the same length."}, status=status.HTTP_400_BAD_REQUEST)
        # Validate skills IDs
        skills_objects = []
        for skill_id in skills_required:
            skill = get_object_or_404(Skill, id=skill_id)
            skills_objects.append(skill)

        # Create Internship instance
        internship = Internship.objects.create(
            fullname=fullname,
            organisation_or_individual=organisation_or_individual,
            profile_title=profile_title,
            email=email,
            job_responsibilities=job_responsibilities,
            designation=designation,
            location=location,
            internship_type=internship_type,
            start_date=start_date,
            end_date=end_date,
            duration=duration,
            salary=salary,
            gender=gender,
            education_required=education_required,
            age_required=age_required,
            skills_experience=skills_experience,
            posted_by=request.user,
            is_active=is_active,
            status=status_field,
            thumbnail=thumbnail,
            logo=logo,
            language=language,
            mobile_number = mobile_number,
        )

        # Add skills_required
        internship.skills_required.set(skills_objects)
        internship.save()

        return Response(
            {"success": True, "message": "Internship created successfully."},
            status=status.HTTP_201_CREATED
        )


class InternshipDetailView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, internship_id):
        try:
            user = request.user
            internship = Internship.objects.get(id=internship_id, is_active=True)
        except Internship.DoesNotExist:
            return Response(
                {"success": False, "message": "Internship not found."},
                status=status.HTTP_404_NOT_FOUND,
            )

        serializer = InternshipViewSerializer(internship)
        create_by = internship.posted_by
        create_by_status = create_by.id == user.id

        save_status = SavedInternship.objects.filter(internship_id = internship_id, user_id = user.id ).exists()
        applied_status = InternshipApplication.objects.filter(internship = internship_id, user =user.id).exists()
        save_count = SavedInternship.objects.filter(user_id = user.id ).count()
        print(save_status,applied_status)
        return Response(
            {"success": True, "data": serializer.data, "created_by":create_by_status,
              "save_status":save_status,
              "applied_status":applied_status,
              "saved_count":save_count,
              },
            status=status.HTTP_200_OK,
        )

# class EditInternshipView(APIView):
#     permission_classes = [IsAuthenticated]

#     def put(self, request, internship_id):
#         try:
#             internship = Internship.objects.get(id=internship_id, posted_by=request.user)
#         except Internship.DoesNotExist:
#             return Response(
#                 {"success": False, "message": "Internship not found or unauthorized action."},
#                 status=status.HTTP_404_NOT_FOUND,
#             )


#         serializer = InternshipSerializer(internship, data=request.data, partial=True)
#         if serializer.is_valid():
#             serializer.save()

#             return Response(
#                 {"success": True, "message": "Internship updated successfully."},
#                 status=status.HTTP_200_OK,
#             )

#         # Return validation errors
#         return Response(
#             {"success": False, "message": "Invalid data.", "errors": serializer.errors},
#             status=status.HTTP_400_BAD_REQUEST,
#         )
class EditInternshipView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, internship_id):
        # Fetch the internship object
        internship = get_object_or_404(Internship, id=internship_id, posted_by=request.user)

        # Pass data to serializer
        serializer = InternshipEditSerializer(instance=internship, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(
                {"success": True, "message": "Internship updated successfully."},
                status=status.HTTP_200_OK,
            )

        # Return validation errors
        return Response(
            {"success": False, "message": "Invalid data.", "errors": serializer.errors},
            status=status.HTTP_400_BAD_REQUEST,
        )

class MyInternshipListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        internships = (Internship.objects.filter(posted_by=user, is_active=True)
        .annotate(applications_count=Count('applications'))
        .order_by('-created_at')
        )


        serializer = InternshipViewSerializer(internships, many=True)
        serializer_data = serializer.data

        if serializer.data:
            return Response({
                "success": True,
                "message": "Internship list fetched successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                "success": False,
                "message": "data not found"
            }, status=status.HTTP_404_NOT_FOUND)

class AppliedInternshipListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        # Fetch applications by the user
        applications = InternshipApplication.objects.filter(user=user).select_related('internship').order_by('-applied_at')

        serializer = AppliedInternshipSerializer(applications, many=True, context={'request': request})
        if serializer.data:
            return Response({
                "success": True,
                "message": "Applied internship list fetched successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                "success": False,
                "message": "data not found"
            }, status=status.HTTP_404_NOT_FOUND)

class UpdateInternshipApplicationStatusView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, internship_id):
        data = request.data
        user_id = data.get('user_id', None)

        try:
            application = InternshipApplication.objects.get(user=user_id, internship_id=internship_id)
        except InternshipApplication.DoesNotExist:
            return Response({
                "success": False,
                "message": "Internship application not found."
            }, status=status.HTTP_404_NOT_FOUND)

        # Serialize and validate the input
        serializer = InternshipApplicationStatusUpdateSerializer(application, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            Notification.objects.create(
                user=application.user,  # Notify the applicant
                action_type="applicant_status",
                message=f"Your application for the internship '{application.internship.title}' has been updated. Current status: {application.status}."
            )
            if application.user.privacy_settings.enable_notifications:
                response = send_push_notification(
                    fcm_token=application.user.fcm_token,
                    title="Application status",
                    body=f"Your application for the internship '{application.internship.title}' has been updated. Current status: {application.status}."
                )
            return Response({
                "success": True,
                "push_notification": response,
                "message": "Internship application status updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "success": False,
            "message": "Invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class ApplyForInternshipView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        serializer = InternshipApplicationSerializer(data=request.data, context={'request': request})
        if serializer.is_valid():
            # Save the application with the current user
            serializer.save(user=request.user)
            return Response({
                "success": True,
                "message": "You have successfully applied for the internship.",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)

        return Response({
            "success": False,
            "message": "Invalid data.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class BaseInternshipApplicationListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, internship_id, status_filter):
        try:
            internship = Internship.objects.get(id=internship_id, posted_by=request.user)
        except Internship.DoesNotExist:
            return Response({
                "success": False,
                "message": "Internship not found or you are not authorized to view applications."
            }, status=status.HTTP_404_NOT_FOUND)
        
        age_limit = request.GET.get('age', None)

        # Filter applications by status
        applications = InternshipApplication.objects.filter(internship=internship, status=status_filter)
        if age_limit not in [None, '']:
            try:
                age_limit = int(age_limit)
                applications = applications.filter(user__profile__age__lte=age_limit)
            except ValueError:
                return Response({
                    "success": False,
                    "message": "Invalid age parameter. Please provide a valid integer value."
                }, status=status.HTTP_400_BAD_REQUEST)

        # Serialize and return the data
        serializer = UserInternshipApplicationSerializer(applications, many=True)
        serializer_data= serializer.data
        for i in range(len(serializer_data)):
            user_data = applications[i].user.profile.location
            print(user_data)
            location = "Unknown Location"
            if user_data:
                parts = user_data.split(',')
                if len(parts) == 2 and parts[0].strip().isdigit() and parts[1].strip().isdigit():
                    city_id, state_id = map(int, parts)  # Convert both parts to integers

                    # Fetch city and state names
                    city = City.objects.filter(id=city_id).first()
                    state = State.objects.filter(id=state_id).first()

                    if city and state:
                        location = f"{city.name}, {state.name}"

            serializer_data[i]["location"]= location
            primary_industry = applications[i].user.profile.selected_primary_industry.name
            serializer_data[i]["primary_industry"]= primary_industry
            primary_skill = applications[i].user.profile.selected_primary_skill.name
            serializer_data[i]["primary_skill"]= primary_skill
            age = applications[i].user.profile.age
            serializer_data[i]["age"]= age
            # description
            description = applications[i].description
            serializer_data[i]["description"]= description
            user_status = applications[i].status
            serializer_data[i]["status"]= user_status
        if serializer.data:
            return Response({
                "success": True,
                "data": serializer.data
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                "success": False,
                "message": "data not found"
            }, status=status.HTTP_404_NOT_FOUND)

class InternshipAppliedUserListView(BaseInternshipApplicationListView):
    def get(self, request, internship_id):
        return super().get(request, internship_id, status_filter='applied')

class InternshipSelectedUserListView(BaseInternshipApplicationListView):
    def get(self, request, internship_id):
        return super().get(request, internship_id, status_filter='selected')


class InternshipWaitingListUserListView(BaseInternshipApplicationListView):
    def get(self, request, internship_id):
        return super().get(request, internship_id, status_filter='waiting_list')

class InternshipSaveLaterUserListView(BaseInternshipApplicationListView):
    def get(self, request, internship_id):
        return super().get(request, internship_id, status_filter='save_later')

class InternshipRejectedUserListView(BaseInternshipApplicationListView):
    def get(self, request, internship_id):
        return super().get(request, internship_id, status_filter='rejected')


class UpdateInternshipStatusView(APIView):
    permission_classes = [IsAuthenticated]

    def put(self, request, internship_id, *args, **kwargs):
        internship = get_object_or_404(Internship, pk=internship_id, posted_by=request.user)
        serializer = InternshipStatusSerializer(instance=internship, data=request.data, partial=True)

        if serializer.is_valid():
            serializer.save()
            return Response({
                "success": "true",
                "message": "Internship status updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "success": "error",
            "message": "Failed to update internship status.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

class UserDetailsView(APIView):

    permission_classes = [IsAuthenticated]



    def get(self, request, application_id, type_of_application):

        # Mapping model classes to application types

        model_mapping = {

            "internship": InternshipApplication,

            "apprenticeship": ApprenticeApplication,

            "castingcall": CastingCallApplication,

            "gigwork": GigWorkApplication

        }



        # Check if the type_of_application is valid

        if type_of_application not in model_mapping:

            return Response(

                {"status": "error", "message": "Invalid application type."},

                status=400

            )



        # Fetch the corresponding application

        application_model = model_mapping[type_of_application]

        try:

            application = application_model.objects.get(id=application_id)

        except application_model.DoesNotExist:

            return Response(

                {"success": False, "message": f"The application with ID {application_id} does not exist in {type_of_application}."},

                status=404

            )

        application = get_object_or_404(application_model, id=application_id)



        # Fetch the user who posted the application

        

        user = application.user

        profile = user.profile



        # Fetch related stats

        posts_count = Post.objects.filter(user=user).count()

        followers_count = Follow.objects.filter(following=user).count()

        likes_count = Like.objects.filter(post__user=user).count()



        # Prepare the response data

        data = {

            "username": user.username,

            "userid": user.id,

            "userimage": profile.profile_image.url if profile.profile_image else None,

            "coverimage": profile.cover_image.url if profile.cover_image else None,

            "biography": profile.bio,

            "posts": posts_count,

            "followers": followers_count,

            "likes": likes_count,

            "skills": [ i.name for i in profile.selected_skills.all()],

            "email_address": user.mobile_or_email if '@' in user.mobile_or_email else None,

            "contact_number": user.mobile_number,

        }

        if type_of_application == 'internship' or type_of_application == 'apprenticeship':

            data['description'] = application.description

        elif type_of_application == 'gigwork':

            data['description'] = application.user_message

            data['attachments'] = application.attachments

        elif type_of_application == 'castingcall':

            data['description'] = application.cover_letter



        return Response({"sucess": True, "data": data}, status=200)

class UserProjectDetails(APIView):
    def get(self, request, user_id):
        try:
            # Get the user
            user = get_object_or_404(CustomUser, pk=user_id)

            # Fetch projects created by the user
            created_projects = Project.objects.filter(user=user)

            # Fetch projects where the user is part of a team
            team_projects = Project.objects.filter(teams__members__user=user)

            # Combine the querysets using Python sets to ensure uniqueness
            all_projects = list(set(created_projects) | set(team_projects))

            # Serialize the projects
            projects_data = []
            for project in all_projects:
                projects_data.append({
                    "project_id":project.id,
                    "project_title": project.project_title,
                    "project_tagline": project.project_tagline,
                    "created_by":project.user_id,
                    "role_played": project.role_played,
                    "language": project.language,
                    "project_type": project.project_type,
                    "progress_percentage": project.progress_percentage,
                    "genre": project.genre,
                    "description": project.description,
                    "created_at": project.created_at,
                    "thumbnail_image": project.thumbnail_image.url if project.thumbnail_image else None,
                    "cover_image": project.cover_image.url if project.cover_image else None,
                    "status": project.status,
                })

            return Response({
                "status": "success",
                "message": "Projects retrieved successfully.",
                "data": projects_data
            }, status=status.HTTP_200_OK)

        except Exception as e:
            return Response({
                "status": "error",
                "message": "An error occurred while retrieving project details.",
                "details": str(e)
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        
class CreditViewSet(viewsets.ViewSet):
    permission_classes = [IsAuthenticated]

    def add_credit(self, request):
        """Add a user to the credit list of a project."""
        project_id = request.data.get('project_id')
        user_id = request.data.get('user_id')
        role = request.data.get('role')
        print(request.user)

        try:
            project = Project.objects.get(id=project_id, user_id=request.user)
        except Project.DoesNotExist:
            return Response({"error": "Project not found or you do not have permission."},
                            status=status.HTTP_403_FORBIDDEN)

        user = CustomUser.objects.filter(id=user_id).first()
        if not user:
            return Response({"error": "User not found."}, status=status.HTTP_404_NOT_FOUND)
        
        existing_team_member = TeamMember.objects.filter(
            team__project=project,
            user=user
        ).exists()
        if existing_team_member:
            return Response({
                "error": "User is already part of a casting crew team for this project."
            }, status=status.HTTP_400_BAD_REQUEST)

        # Add credit
        credit, created = Credit.objects.get_or_create(project=project, user=user, defaults={'role': role})
        if not created:
            return Response({"error": "User is already in the credit list."}, status=status.HTTP_400_BAD_REQUEST)
        
        Notification.objects.create(
            user=user,  # Notify the user added to the project
            sender=request.user,
            action_type="added_to_credit",
            message=f"You have been added to the credit list of the project {project.project_title} as {role}."
        )
        privacy_settings, created = PrivacySettings.objects.get_or_create(user=user)
        # if user.privacy_settings.enable_notifications:
        if privacy_settings.enable_notifications:
            response = send_push_notification(
                    fcm_token= user.fcm_token,
                    title="Added to credit",
                    body= f"You have been added to the credit list of the project {project.project_title} as {role}."
                )

        return Response({"message": "Credit added successfully."}, status=status.HTTP_201_CREATED)

    def remove_credit(self, request, credit_id):
        """Remove a user from the credit list of a project."""
        try:
            credit = Credit.objects.get(id=credit_id, project__user_id=request.user)
        except Credit.DoesNotExist:
            return Response({"error": "Credit entry not found or you do not have permission."},
                            status=status.HTTP_403_FORBIDDEN)

        credit.delete()
        return Response({"message": "Credit removed successfully."}, status=status.HTTP_200_OK)

    def manage_credit_request(self, request, credit_id):
        action = request.data.get('action')
        if action not in ['accept', 'decline']:
            return Response({"error": "Invalid action."}, status=status.HTTP_400_BAD_REQUEST)

        try:
            credit = Credit.objects.get(id=credit_id, user=request.user)
        except Credit.DoesNotExist:
            return Response({"error": "Credit entry not found."}, status=status.HTTP_404_NOT_FOUND)

        if action == 'accept':
            credit.status = 'accepted'
            credit.save()
            Notification.objects.filter(user=request.user, action_type="credit_request").delete()
            return Response({"message": "Credit request accepted."}, status=status.HTTP_200_OK)
        elif action == 'decline':
            credit.status = 'declined'
            credit.save()
            Notification.objects.filter(user=request.user, action_type="credit_request").delete()
            return Response({"message": "Credit request declined."}, status=status.HTTP_200_OK)

    def exit_credit(self, request, credit_id):
        """Allows a user to exit from a credit list."""
        try:
            credit = Credit.objects.get(id=credit_id, user=request.user, status='accepted')
        except Credit.DoesNotExist:
            return Response({"error": "Credit entry not found or not accepted."}, status=status.HTTP_404_NOT_FOUND)

        credit.delete()
        return Response({"message": "You have exited the credit list."}, status=status.HTTP_200_OK)
    
    @action(detail=False, methods=['post'], url_path='request-credit')
    def request_credit(self, request):
        project_id = request.data.get('project_id')
        role = request.data.get('role')
        comment = request.data.get('comment')

        if not project_id or not role:
            return Response({"error": "Project ID and role are required."}, status=status.HTTP_400_BAD_REQUEST)

        try:
            project = Project.objects.get(id=project_id)
        except Project.DoesNotExist:
            return Response({"error": "Project not found."}, status=status.HTTP_404_NOT_FOUND)
        
        existing_team_member = TeamMember.objects.filter(
            team__project=project,
            user=request.user
        ).exists()
        if existing_team_member:
            return Response({
                "error": "You are already part of a casting crew team for this project. Request for a credit instead."
            }, status=status.HTTP_400_BAD_REQUEST)

        # Check if the user already has a pending or accepted credit for the project
        existing_credit = Credit.objects.filter(project=project, user=request.user).exclude(status='declined').first()
        if existing_credit:
            return Response({"error": "You already have a pending or accepted credit request for this project."},
                            status=status.HTTP_400_BAD_REQUEST)

        # Create a new credit request
        credit = Credit.objects.create(
            project=project,
            user=request.user,
            role=role,
            comment=comment,
            status='pending'
        )
        Notification.objects.create(
            user=project.user,  # Notify the project owner
            sender=request.user,
            action_type="credit_request",
            message=f"{request.user.full_name} has requested to join the credits of the project {project.project_title} as {role}."
        )
        if project.user.privacy_settings.enable_notifications:
            response = send_push_notification(
                    fcm_token=project.user.fcm_token,
                    title="credit request",
                    body=f"{request.user.full_name} has requested to join the credits of the project {project.project_title} as {role}."
                )
        else:
            response = None

        return Response({
            "message": "Your credit request has been submitted and is pending approval.",
            "data": {
                "project_id": project.id,
                "project_name": project.project_title,
                "role": credit.role,
                "status": credit.status
            }
        }, status=status.HTTP_201_CREATED)
    
class ProjectCreditsAPIView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, project_id):
        project = get_object_or_404(Project, pk=project_id)

        # Fetch credits related to the project
        credits = Credit.objects.filter(project=project, status="accepted")

        # Prepare response data
        credit_data = []
        for credit in credits:
            user_profile = Profile.objects.filter(user=credit.user).first()
            credit_data.append({
                "user_id": credit.user.id,
                "username": credit.user.username,
                "full_name": credit.user.full_name,
                "role": credit.role,
                "profile_image": user_profile.profile_image.url if user_profile  else None
            })

        return Response({"credits": credit_data}, status=status.HTTP_200_OK)

class UserProjectsView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, user_id):
        try:
            created_projects = Project.objects.filter(user_id=user_id)

            credited_projects = Project.objects.filter(credits__user_id=user_id).distinct()

            all_projects = created_projects.union(credited_projects)

            serializer = ProjectSerializer(all_projects, many=True, context={'request': request})

            return Response({
                "user_id": user_id,
                "projects": serializer.data
            }, status=status.HTTP_200_OK)
        except Exception as e:
            return Response({"error": f"An error occurred: {str(e)}"}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
    
class CastingCallDetailAPIView(APIView):
    def get(self, request, casting_call_id):
        casting_call = get_object_or_404(CastingCall, id=casting_call_id)
        serializer = CastingCallSerializer(casting_call)
        experience = serializer.data.get('experience','')
        formatted_experience = []
        if experience:
            experience_list = experience.split(',')
            formatted_experience = [f"{exp.strip()} years of experience" for exp in experience_list]
        saved = SavedCastingCall.objects.filter(
            user=request.user, casting_call=casting_call
        ).exists()

        # Check if the current user has applied for this casting call
        applied = CastingCallApplication.objects.filter(
            user=request.user, casting_call=casting_call
        ).exists()
        created_by = True
        created_by_user = serializer.data.get('user_id')
        if created_by_user == request.user.id:
            created_by = True
        else:
            created_by= False

        # Add additional fields to the serialized data
        response_data = serializer.data
        response_data['saved_status'] = saved
        response_data['applied_status'] = applied
        response_data['formatted_experience']= formatted_experience
        response_data['created_by']= created_by



        return Response(response_data, status=status.HTTP_200_OK)
    
class ProjectListAPI(APIView):
    def get(self, request, *args, **kwargs):
        projects = Project.objects.all()
        serializer = ProjectSerializer(projects,many=True, context={"request": request})
        return Response({"sucess": True, "data":serializer.data}, status=200)
    
class CastingCallListAPI(APIView):
    def get(self, request, *args, **kwargs):
        casting_calls = CastingCall.objects.filter(deleted=False)
        serializer = CastingCallSerializer(casting_calls, many=True, context={"request": request})
        return Response({"success": True, "data": serializer.data}, status=200)
    
class InternshipListAPI(APIView):
    def get(self, request, *args, **kwargs):
        internship_call = Internship.objects.all()
        serializer = InternshipSerializer(internship_call,many=True, context={"request": request})
        return Response({"sucess": True, "data":serializer.data}, status=200)
    
class ApprenticeshipListAPI(APIView):
    def get(self, request, *args, **kwargs):
        apprenticeship_call = Apprenticeship.objects.all()
        serializer = ApprenticeshipSerializer(apprenticeship_call,many=True, context={"request": request})
        return Response({"sucess": True, "data":serializer.data}, status=200)
    
class GigWorkListAPI(APIView):
    def get(self, request, *args, **kwargs):
        gigwork_call = GigWork.objects.all()
        serializer = GigWorkSerializer(gigwork_call,many=True, context={"request": request})
        return Response({"sucess": True, "data":serializer.data}, status=200)
    
class CastingCallApplicantsAPI(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request, casting_call_id, status_filter):
        try:
            casting_call = CastingCall.objects.get(id=casting_call_id)
        except CastingCall.DoesNotExist:
            return Response(
                {"error": "Casting call not found."},
                status=status.HTTP_404_NOT_FOUND
            )

        # Validate status filter
        valid_statuses = ['applied', 'selected', 'waiting_list', 'save_later', 'rejected']
        if status_filter not in valid_statuses:
            return Response(
                {"error": f"Invalid status. Valid statuses are: {', '.join(valid_statuses)}"},
                status=status.HTTP_400_BAD_REQUEST
            )

        # Fetch applications with the given status
        applications = CastingCallApplication.objects.filter(
            casting_call=casting_call,
            status=status_filter
        ).select_related('user')

        # Serialize users
        users = [app.user for app in applications]
        serialized_users = CustomUserSerializer(users, many=True)

        # return Response(serialized_users.data, status=status.HTTP_200_OK)
        return Response({"sucess": True, "data":serialized_users.data}, status=200)
    
class SoftDeleteCastingCallView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request, casting_call_id):
        try:
            casting_call = CastingCall.objects.get(id=casting_call_id, user=request.user)
            casting_call.deleted = True
            casting_call.save()

            return Response({
                "status": "success",
                "message": "Casting call deleted successfully."
            }, status=status.HTTP_200_OK)

        except CastingCall.DoesNotExist:
            return Response({
                "status": "error",
                "message": "Casting call not found or you are not authorized to delete it."
            }, status=status.HTTP_404_NOT_FOUND)
        
class AppliedByMeCastingCallAPI(APIView):
    """
    API to list casting calls applied by the logged-in user.
    """
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        user = request.user
        applications = CastingCallApplication.objects.filter(user=user)
        casting_calls = CastingCall.objects.filter(applications__in=applications).distinct()

        serializer = CastingCallSerializer(casting_calls, many=True, context={"request": request})
        return Response({
            "success": True,
            "data": serializer.data
        }, status=200)


class CreatedByMeCastingCallAPI(APIView):
    """
    API to list casting calls created by the logged-in user.
    """
    permission_classes = [IsAuthenticated]

    def get(self, request, *args, **kwargs):
        user = request.user
        casting_calls = CastingCall.objects.filter(user=user, deleted=False)  # Exclude deleted ones

        serializer = CastingCallSerializer(casting_calls, many=True, context={"request": request})
        return Response({
            "success": True,
            "data": serializer.data
        }, status=200)
    
class AppliedGigWorkListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        applications = GigWorkApplication.objects.filter(user=user).select_related('gigwork')
        serializer = GigWorkApplicationSerializer(applications, many=True, context={'request': request})
        return Response({
            "success": True,
            "applied_gigworks": serializer.data
        })

class CreatedGigWorkListView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        gigworks = GigWork.objects.filter(user=user)
        serializer = GigWorkSerializer(gigworks, many=True, context={'request': request})
        return Response({
            "success": True,
            "created_gigworks": serializer.data
        })

class ShareGigWorkView(APIView):
    def post(self, request):
        gigwork_id = request.data.get('gigwork_id')
        if not gigwork_id:
            return Response({"error": "Gigwork ID is required."}, status=status.HTTP_400_BAD_REQUEST)
        gigwork = get_object_or_404(GigWork, id=gigwork_id)

        # Generate the event detail URL
        base_url = request.build_absolute_uri('/')
        gigwork_detail_url = f"{base_url}api/gigworks/details/{gigwork_id}"

        return Response(
            {"message": "Shareable URL generated successfully.", "url": gigwork_detail_url},
            status=status.HTTP_200_OK
        )
    
class ShareCastingCallView(APIView):
    def post(self, request):
        castingcall_id = request.data.get('castingcall_id')
        if not castingcall_id:
            return Response({"error": "Casting Call ID is required."}, status=status.HTTP_400_BAD_REQUEST)
        castingcall = get_object_or_404(CastingCall, id=castingcall_id)

        # Generate the event detail URL
        base_url = request.build_absolute_uri('/')
        castingcall_detail_url = f"{base_url}api/casting-call/{castingcall_id}/"

        return Response(
            {"message": "Shareable URL generated successfully.", "url": castingcall_detail_url},
            status=status.HTTP_200_OK
        )
    
class ShareProjectView(APIView):
    def post(self, request):
        project_id = request.data.get('project_id')
        if not project_id:
            return Response({"error": "Project ID is required."}, status=status.HTTP_400_BAD_REQUEST)
        event = get_object_or_404(Project, id=project_id)

        # Generate the event detail URL
        base_url = request.build_absolute_uri('/')
        project_detail_url = f"{base_url}api/project/{project_id}/"

        return Response(
            {"message": "Shareable URL generated successfully.", "url": project_detail_url},
            status=status.HTTP_200_OK
        )
    
class ShareInternshipView(APIView):
    def post(self, request):
        internship_id = request.data.get('internship_id')
        if not internship_id:
            return Response({"error": "Internship ID is required."}, status=status.HTTP_400_BAD_REQUEST)
        event = get_object_or_404(Internship, id=internship_id)

        # Generate the event detail URL
        base_url = request.build_absolute_uri('/')
        internship_detail_url = f"{base_url}api/internship/{internship_id}/"

        return Response(
            {"message": "Shareable URL generated successfully.", "url": internship_detail_url},
            status=status.HTTP_200_OK
        )
    
class ShareApprenticeshipView(APIView):
    def post(self, request):
        apprenticeship_id = request.data.get('apprenticeship_id')
        if not apprenticeship_id:
            return Response({"error": "Apprenticeship ID is required."}, status=status.HTTP_400_BAD_REQUEST)
        event = get_object_or_404(Apprenticeship, id=apprenticeship_id)

        # Generate the event detail URL
        base_url = request.build_absolute_uri('/')
        apprenticeship_detail_url = f"{base_url}api/apprenticeship/{apprenticeship_id}/"

        return Response(
            {"message": "Shareable URL generated successfully.", "url": apprenticeship_detail_url},
            status=status.HTTP_200_OK
        )